@@ -21,14 +21,14 @@
 #include <sys/wait.h>
 #include <sys/stat.h>
 
-#include "cs_types.h"
-#include "cs.h"
-#include "cistpl.h"
-#include "ds.h"
+#include <pcmcia/cs_types.h>
+#include <pcmcia/cs.h>
+#include <pcmcia/cistpl.h>
+#include <pcmcia/ds.h>
 
 #if 0
 static const char *version =
-"cardinfo.c 1.14 1996/08/06 03:15:14 (David Hinds)";
+"cardinfo.c 1.15 1996/11/26 07:25:29 (David Hinds)";
 #endif
 
 /*====================================================================*/
@@ -41,6 +41,8 @@ static char *config = "/etc/pcmcia";
 
 static char *scheme = "/var/run/pcmcia-scheme";
 
+static char *stabfile = "/var/run/stab";
+
 /*====================================================================*/
 
 static int lookup_dev(char *name)
@@ -377,7 +379,8 @@ void usage(char *name)
 {
     int i;
     fprintf(stderr, "usage: %s command [socket #]\n", name);
-    fprintf(stderr, "    or %s scheme [scheme name]\n", name);
+    fprintf(stderr, "    or %s [-c config] [-f scheme]"
+	    " [-s stab] scheme [name]\n", name);
     fprintf(stderr, "  commands:");
     for (i = 0; i < NCMD; i++)
 	fprintf(stderr, " %s", cmdname[i]);
@@ -392,6 +395,25 @@ void usage(char *name)
 void main(int argc, char *argv[])
 {
     int major, cmd, fd[MAX_SOCKS], ns, ret, i;
+    int optch, errflg;
+    extern char *optarg;
+
+    errflg = 0;
+    while ((optch = getopt(argc, argv, "c:f:s:")) != -1) {
+	switch (optch) {
+	case 'c':
+	    config = strdup(optarg); break;
+	case 'f':
+	    scheme = strdup(optarg); break;
+	case 's':
+	    stabfile = strdup(optarg); break;
+	default:
+	    errflg = 1; break;
+	}
+    }
+    
+    if (errflg || (argc < 2) || (argc > 3))
+	usage(argv[0]);
 
     if (geteuid() != 0) {
         fprintf(stderr, "cardctl must be setuid root\n");
@@ -404,9 +426,6 @@ void main(int argc, char *argv[])
 	exit(EXIT_FAILURE);
     }
 
-    if ((argc < 2) || (argc > 3))
-	usage(argv[0]);
-
     if (strcmp(argv[1], "scheme") == 0) {
 	setuid(getuid());
 	if (do_scheme((argc == 2) ? NULL : argv[2]) == 0)
