@@ -25,15 +25,15 @@
 #include <forms.h>
 
 #undef Status
-#include "version.h"
-#include "cs_types.h"
-#include "cs.h"
-#include "cistpl.h"
-#include "ds.h"
+#include <pcmcia/version.h>
+#include <pcmcia/cs_types.h>
+#include <pcmcia/cs.h>
+#include <pcmcia/cistpl.h>
+#include <pcmcia/ds.h>
 
 #if 0
 static const char *version =
-"cardinfo.c 1.10 1996/08/06 05:07:07 (David Hinds)";
+"cardinfo.c 1.12 1996/12/08 20:06:27 (David Hinds)";
 #endif
 
 /*====================================================================*/
@@ -442,12 +442,12 @@ void main(int argc, char *argv[])
 
     if (ioctl(st[0].fd, DS_GET_CARD_SERVICES_INFO, &serv) == 0) {
 	if (serv.Revision != CS_RELEASE_CODE) {
-	    fprintf(stderr, "Card Services release does not match!");
+	    fprintf(stderr, "Card Services release does not match!\n");
 	    exit(EXIT_FAILURE);
 	}
     }
     else {
-	fprintf(stderr, "could not get CS revision info!");
+	fprintf(stderr, "could not get CS revision info!\n");
 	exit(EXIT_FAILURE);
     }
     
