@@ -24,12 +24,12 @@
 #include <sys/kd.h>
 #include <sys/file.h>
 
-#include "version.h"
-#include "config.h"
-#include "cs_types.h"
-#include "cs.h"
-#include "cistpl.h"
-#include "ds.h"
+#include <pcmcia/version.h>
+#include <pcmcia/config.h>
+#include <pcmcia/cs_types.h>
+#include <pcmcia/cs.h>
+#include <pcmcia/cistpl.h>
+#include <pcmcia/ds.h>
 
 #define VERSION(v,p,s) (((v)<<16)+(p<<8)+s)
 #if (LINUX_VERSION_CODE > VERSION(1,3,0))
@@ -45,7 +45,7 @@
 #include "cardmgr.h"
 
 static const char *version =
-    "cardmgr.c 1.80 1996/08/06 05:06:29 (David Hinds)";
+    "cardmgr.c 1.84 1996/12/07 05:09:35 (David Hinds)";
 
 /*====================================================================*/
 
@@ -104,6 +104,9 @@ static char *stabfile = "/var/run/stab";
 /* If set, don't generate beeps when cards are inserted */
 static int be_quiet = 0;
 
+/* If set, use modprobe instead of insmod */
+static int do_modprobe = 0;
+
 /* most recent signal */
 static int caught_signal = 0;
 
@@ -470,8 +473,17 @@ static void free_config(void)
 static int execute(char *msg, char *cmd)
 {
     int ret;
+    FILE *f;
+    char line[256];
+    
     syslog(LOG_INFO, "executing: '%s'", cmd);
-    ret = system(cmd);
+    strcat(cmd, " 2>&1");
+    f = popen(cmd, "r");
+    while (fgets(line, 255, f)) {
+	line[strlen(line)-1] = '\0';
+	syslog(LOG_INFO, "+ %s", line);
+    }
+    ret = pclose(f);
     if (WIFEXITED(ret)) {
 	if (WEXITSTATUS(ret))
 	    syslog(LOG_INFO, "%s exited with status %d",
@@ -533,20 +545,24 @@ static int install_module(char *mod, char *opts)
     }
     ml->usage++;
     if (ml->usage == 1) {
-	if (strchr(mod, '/') != NULL)
-	    sprintf(path, "%s/%s.o", modpath, mod);
-	else
-	    sprintf(path, "%s/pcmcia/%s.o", modpath, mod);
-	if (access(path, R_OK) != 0) {
-	    syslog(LOG_INFO, "module %s not available", path);
-	    return -1;
+	if (do_modprobe) {
+	    sprintf(cmd, "/sbin/modprobe %s", mod);
+	} else {
+	    if (strchr(mod, '/') != NULL)
+		sprintf(path, "%s/%s.o", modpath, mod);
+	    else
+		sprintf(path, "%s/pcmcia/%s.o", modpath, mod);
+	    if (access(path, R_OK) != 0) {
+		syslog(LOG_INFO, "module %s not available", path);
+		return -1;
+	    }
+	    sprintf(cmd, "/sbin/insmod %s", path);
 	}
-	sprintf(cmd, "/sbin/insmod %s", path);
 	if (opts) {
 	    strcat(cmd, " ");
 	    strcat(cmd, opts);
 	}
-	return execute("insmod", cmd);
+	return execute(do_modprobe ? "modprobe" : "insmod", cmd);
     }
     else
 	return 0;
@@ -1024,17 +1040,19 @@ static int init_sockets(void)
 void main(int argc, char *argv[])
 {
     int optch, errflg;
-    int i, ret, event;
+    int i, max_fd, ret, event;
     int verbose = 0;
     extern char *optarg;
     fd_set fds;
     FILE *f;
 
     errflg = 0;
-    while ((optch = getopt(argc, argv, "qvc:m:p:s:")) != -1) {
+    while ((optch = getopt(argc, argv, "qvdc:m:p:s:")) != -1) {
 	switch (optch) {
 	case 'q':
 	    be_quiet = 1; break;
+	case 'd':
+	    do_modprobe = 1; break;
 	case 'v':
 	    verbose = 1; break;
 	case 'c':
@@ -1050,8 +1068,9 @@ void main(int argc, char *argv[])
 	}
     }
     if (errflg || (optind < argc)) {
-	fprintf(stderr, "usage: %s [-q] [-v] [-c configpath] "
-		"[-m modpath] [-p pidfile] [-s stabfile]\n", argv[0]);
+	fprintf(stderr, "usage: %s [-q] [-v] [-d] [-c config] "
+		"[-m modpath]\n               [-p pidfile] "
+		"[s stabfile]\n", argv[0]);
 	exit(EXIT_FAILURE);
     }
 
@@ -1114,12 +1133,15 @@ void main(int argc, char *argv[])
     if (signal(SIGPWR, catch_signal) == SIG_ERR)
 	syslog(LOG_INFO, "signal(SIGPWR): %m");
 
+    for (i = max_fd = 0; i < sockets; i++)
+	max_fd = (socket[i].fd > max_fd) ? socket[i].fd : max_fd;
+    
     for (;;) {
 	FD_ZERO(&fds);
 	for (i = 0; i < sockets; i++)
 	    FD_SET(socket[i].fd, &fds);
 
-	while ((ret = select(MAX_SOCKS+4, &fds, NULL, NULL, NULL)) < 0) {
+	while ((ret = select(max_fd+1, &fds, NULL, NULL, NULL)) < 0) {
 	    if (errno == EINTR)
 		handle_signal();
 	    else {
