@@ -12,21 +12,77 @@
 #include <unistd.h>
 #include <string.h>
 #include <errno.h>
+#include <fcntl.h>
 
 #include <asm/io.h>
 
+#include <pcmcia/config.h>
 #include "i82365.h"
 #include "pd67xx.h"
 #include "vg468.h"
 #include "tcic.h"
 
 static const char *version =
-"probe.c 1.18 1996/09/25 05:22:13 (David Hinds)\n";
+"probe.c 1.20 1996/12/06 16:51:34 (David Hinds)\n";
 
 static int i365_base = 0x03e0;
 
 /*====================================================================*/
 
+#ifdef CONFIG_PCI
+
+typedef struct {
+    char	*tag;
+    char	*name;
+} pci_id_t;
+
+pci_id_t pci_id[] = {
+    { "Cirrus Logic CL 6729", "Cirrus PD6729" },
+    { "Vendor id=1180. Device id=466.", "Ricoh RL5C466" },
+    { "Vendor id=10b3. Device id=b106.", "SMC 34C90" },
+    { "Vendor id=1013. Device id=1110.", "Cirrus PD6832" },
+    { "Vendor id=104c. Device id=ac12.", "TI 1131" }
+};
+#define PCI_COUNT (sizeof(pci_id)/sizeof(pci_id_t))
+
+static int pci_probe(int verbose, int module)
+{
+    char buf[4096], *name = NULL;
+    int sock = 0, fd, nb, i;
+    
+    if (!module)
+	printf("PCI bridge probe: ");
+
+    fd = open("/proc/pci", O_RDONLY);
+    if (fd != -1) {
+	nb = read(fd, buf, 4096);
+	buf[nb] = '\0';
+	for (i = 0; i < PCI_COUNT; i++)
+	    if (strstr(buf, pci_id[i].tag) != NULL)
+		break;
+	if (i != PCI_COUNT) {
+	    name = pci_id[i].name;
+	    sock = 2;
+	}
+	close(fd);
+    }
+    
+    if (sock != 0) {
+	if (module)
+	    printf("i82365\n");
+	else
+	    printf("%s found, %d sockets.\n", name, sock);
+	return 0;
+    } else {
+	if (!module)
+	    printf("not found.\n");
+	return -ENODEV;
+    }
+}
+#endif
+
+/*====================================================================*/
+
 static u_char i365_get(u_short sock, u_short reg)
 {
     u_short port = I365_PORT(sock);
@@ -70,7 +126,7 @@ int i365_probe(int verbose, int module)
     sock = done = 0;
     ioperm(i365_base, 4, 1);
     ioperm(0x80, 1, 1);
-    for (; sock < 8; sock++) {
+    for (; sock < 2; sock++) {
 	val = i365_get(sock, I365_IDENT);
 	if (verbose)
 	    printf("  ident(%d)=%#2.2x", sock, val); 
@@ -93,7 +149,6 @@ int i365_probe(int verbose, int module)
 	    done = 1;
 	}
 	if (done) break;
-	i365_set(sock, I365_MEM(3)+I365_W_OFF, sock);
     }
 
     if (verbose) printf("\n  ");
@@ -105,10 +160,6 @@ int i365_probe(int verbose, int module)
 
     if ((sock == 2) && (strcmp(name, "VLSI 82C146") == 0))
 	name = "i82365sl DF";
-    
-    /* Check for bogus clones that ignore top bit of index register */
-    if ((sock == 4) && (i365_get(0, I365_MEM(3)+I365_W_OFF) == 2))
-	sock = 2;
 
     /* Check for Vadem chips */
     outb_p(0x0e, I365_PORT(0));
@@ -141,7 +192,7 @@ int i365_probe(int verbose, int module)
     if (module)
 	printf("i82365\n");
     else
-	printf("%s found, %d sockets\n", name, sock);
+	printf("%s found, %d sockets.\n", name, sock);
     return 0;
     
 } /* i365_probe */
@@ -265,7 +316,7 @@ int tcic_probe(int verbose, int module, u_long base)
 	default:
 	    printf("Unknown TCIC-2 ID 0x%02x", id);
 	}
-	printf(" found at %#6lx, %d sockets\n", base, sock);
+	printf(" found at %#6lx, %d sockets.\n", base, sock);
     }
     return 0;
     
@@ -301,12 +352,15 @@ void main(int argc, char *argv[])
 
     if (verbose) printf("%s", version);
 
+#ifdef CONFIG_PCI
+    if (pci_probe(verbose, module) == 0)
+	exit(EXIT_SUCCESS);
+    else
+#endif
     if (i365_probe(verbose, module) == 0)
 	exit(EXIT_SUCCESS);
-    else {
-	if (tcic_probe(verbose, module, tcic_base) == 0)
-	    exit(EXIT_SUCCESS);
-	else
-	    exit(EXIT_FAILURE);
-    }
+    else if (tcic_probe(verbose, module, tcic_base) == 0)
+	exit(EXIT_SUCCESS);
+    else
+	exit(EXIT_FAILURE);
 }
