@@ -10,7 +10,7 @@ static char yysccsid[] = "@(#)yaccpar	1.9 (Berkeley) 02/21/93";
 #define YYPREFIX "yy"
 #line 2 "yacc_config.y"
 /*
- * yacc_config.y 1.27 1996/04/29 01:46:11 (David Hinds)\n"
+ * yacc_config.y 1.28 1996/11/26 07:25:29 (David Hinds)\n"
  */
     
 #include <stdlib.h>
@@ -20,10 +20,10 @@ static char yysccsid[] = "@(#)yaccpar	1.9 (Berkeley) 02/21/93";
 #include <syslog.h>
 #include <sys/types.h>
 
-#include "cs_types.h"
-#include "cs.h"
-#include "cistpl.h"
-#include "ds.h"
+#include <pcmcia/cs_types.h>
+#include <pcmcia/cs.h>
+#include <pcmcia/cistpl.h>
+#include <pcmcia/ds.h>
     
 #include "cardmgr.h"
 
