@@ -17,13 +17,13 @@
 #include <sys/ioctl.h>
 #include <sys/stat.h>
 
-#include "cs_types.h"
-#include "cs.h"
-#include "cistpl.h"
-#include "ds.h"
+#include <pcmcia/cs_types.h>
+#include <pcmcia/cs.h>
+#include <pcmcia/cistpl.h>
+#include <pcmcia/ds.h>
 
 static const char *version =
-    "dump_tuples.c v1.22: 1996/04/08 05:00:48 (David Hinds)\n";
+    "dump_tuples.c v1.23: 1996/11/26 07:32:27 (David Hinds)\n";
 
 /*====================================================================*/
 
