@@ -15,12 +15,8 @@
 
 ======================================================================*/
 
-#include "config.h"
-#include "k_compat.h"
-
-#ifdef MODULE
-#define init_3c589_cs init_module
-#endif
+#include <pcmcia/config.h>
+#include <pcmcia/k_compat.h>
 
 #include <linux/kernel.h>
 #include <linux/sched.h>
@@ -41,13 +37,13 @@
 #include <linux/if_arp.h>
 #include <linux/ioport.h>
 
-#include "version.h"
-#include "cs_types.h"
-#include "cs.h"
-#include "cistpl.h"
-#include "cisreg.h"
-#include "ciscode.h"
-#include "ds.h"
+#include <pcmcia/version.h>
+#include <pcmcia/cs_types.h>
+#include <pcmcia/cs.h>
+#include <pcmcia/cistpl.h>
+#include <pcmcia/cisreg.h>
+#include <pcmcia/ciscode.h>
+#include <pcmcia/ds.h>
 
 /* To minimize the size of the driver source I only define operating
    constants if they are used several times.  You'll need the manual
@@ -107,7 +103,7 @@ static char *if_names[] = { "Auto", "10baseT", "10base2", "AUI" };
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
 static char *version =
-"3c589_cs.c 1.84 1996/08/16 05:01:58 (David Hinds)";
+"3c589_cs.c 1.86 1996/12/21 19:30:42 (David Hinds)";
 #endif
 
 #ifdef TC589_DEBUG
@@ -1122,7 +1118,7 @@ static int el3_close(struct device *dev)
 
 /*====================================================================*/
 
-int init_3c589_cs(void)
+int init_module(void)
 {
     servinfo_t serv;
 #ifdef PCMCIA_DEBUG
@@ -1139,7 +1135,6 @@ int init_3c589_cs(void)
     return 0;
 }
 
-#ifdef MODULE
 void cleanup_module(void)
 {
 #ifdef PCMCIA_DEBUG
@@ -1150,4 +1145,3 @@ void cleanup_module(void)
     while (dev_list != NULL)
 	tc589_detach(dev_list);
 }
-#endif /* MODULE */
