@@ -9,12 +9,8 @@
     
 ======================================================================*/
 
-#include "config.h"
-#include "k_compat.h"
-
-#ifdef MODULE
-#define init_aha152x_cs init_module
-#endif
+#include <pcmcia/config.h>
+#include <pcmcia/k_compat.h>
 
 #include <linux/kernel.h>
 #include <linux/sched.h>
@@ -46,16 +42,16 @@
 #define aha152x_reset(ptr) aha152x_reset(ptr, 0)
 #endif
 
-#include "version.h"
-#include "cs_types.h"
-#include "cs.h"
-#include "cistpl.h"
-#include "ds.h"
+#include <pcmcia/version.h>
+#include <pcmcia/cs_types.h>
+#include <pcmcia/cs.h>
+#include <pcmcia/cistpl.h>
+#include <pcmcia/ds.h>
 
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
 static char *version =
-"aha152x_cs.c 1.20 1996/08/29 04:38:47 (David Hinds)";
+"aha152x_cs.c 1.24 1996/12/21 19:30:43 (David Hinds)";
 #endif
 
 /*====================================================================*/
@@ -204,7 +200,7 @@ static void aha152x_config_x(dev_link_t *link)
     scsi_info_t *info;
     tuple_t tuple;
     cisparse_t parse;
-    int i, ints[3];
+    int i, ints[8];
     u_char tuple_data[64];
 #ifdef GET_SCSI_INFO
     Scsi_Device *dev;
@@ -239,9 +235,7 @@ static void aha152x_config_x(dev_link_t *link)
     }
 
     /* Configure card */
-#ifdef MODULE
     driver_template.usage_count = &mod_use_count_;
-#endif
     link->state |= DEV_CONFIG;
 
     do {
@@ -294,7 +288,11 @@ static void aha152x_config_x(dev_link_t *link)
     release_region(link->io.BasePort1, link->io.NumPorts1);
 
     /* Set configuration options for the aha152x driver */
+#if (LINUX_VERSION_CODE >= VERSION(1,3,0))
     ints[0] = 7;
+#else
+    ints[0] = 5;
+#endif
     ints[1] = link->io.BasePort1;
     ints[2] = link->irq.AssignedIRQ;
     ints[3] = host_id;
@@ -442,7 +440,7 @@ static int aha152x_event(event_t event, int priority,
 
 /*====================================================================*/
 
-int init_aha152x_cs(void) {
+int init_module(void) {
     servinfo_t serv;
 #ifdef PCMCIA_DEBUG
     if (pc_debug)
@@ -458,7 +456,6 @@ int init_aha152x_cs(void) {
     return 0;
 }
 
-#ifdef MODULE
 void cleanup_module(void) {
 #ifdef PCMCIA_DEBUG
     if (pc_debug)
@@ -468,4 +465,3 @@ void cleanup_module(void) {
     while (dev_list != NULL)
 	aha152x_detach(dev_list);
 }
-#endif
