@@ -6,12 +6,8 @@
     
 ======================================================================*/
 
-#include "config.h"
-#include "k_compat.h"
-
-#ifdef MODULE
-#define init_cs init_module
-#endif
+#include <pcmcia/config.h>
+#include <pcmcia/k_compat.h>
 
 #include <linux/kernel.h>
 #include <linux/string.h>
@@ -25,17 +21,16 @@
 #include <linux/delay.h>
 #include <asm/system.h>
 
-#include "version.h"
-#include "cs_types.h"
-#include "ss.h"
-
 #define IN_CARD_SERVICES
-#include "cs.h"
-#include "rsrc_mgr.h"
-#include "bulkmem.h"
+#include <pcmcia/version.h>
+#include <pcmcia/cs_types.h>
+#include <pcmcia/ss.h>
+#include <pcmcia/cs.h>
+#include <pcmcia/bulkmem.h>
+#include <pcmcia/cistpl.h>
+#include <pcmcia/cisreg.h>
 #include "cs_internal.h"
-#include "cistpl.h"
-#include "cisreg.h"
+#include "rsrc_mgr.h"
 
 #ifdef CONFIG_APM
 #include <linux/apm_bios.h>
@@ -45,7 +40,7 @@ static int handle_apm_event(apm_event_t event);
 #ifdef PCMCIA_DEBUG
 int pc_debug = PCMCIA_DEBUG;
 static const char *version =
-"cs.c 1.138 1996/08/06 03:11:49 (David Hinds)";
+"cs.c 1.141 1996/12/21 19:30:44 (David Hinds)";
 #endif
 
 static const char *release =
@@ -70,9 +65,11 @@ static struct symbol_table cs_symtab = {
 /* Parameters that can be set with 'insmod' */
 
 static int setup_delay		= HZ/20;	/* ticks */
+static int resume_delay		= HZ/5;		/* ticks */
 static int shutdown_delay	= HZ/40;	/* ticks */
 static int vcc_settle		= HZ*0.3;	/* ticks */
 static int reset_time		= 10;		/* usecs */
+static int unreset_delay	= HZ/10;	/* ticks */
 static int unreset_check	= HZ/10;	/* ticks */
 static int unreset_limit	= 30;		/* unreset_check's */
 
@@ -262,12 +259,11 @@ int register_ss_entry(int nsock, ss_entry_t ss_entry)
 	s->setup.function = &setup_socket;
 	s->shutdown.data = sockets;
 	s->shutdown.function = &shutdown_socket;
-	s->unreset.data = sockets;
-	s->unreset.function = &unreset_socket;
 	/* base address = 0, map = 0 */
 	s->cis_mem.flags = MAP_ACTIVE;
 	s->cis_mem.speed = cis_speed;
 	s->erase_busy.next = s->erase_busy.prev = &s->erase_busy;
+	init_waitqueue(&s->mtd_ready);
 	
 	for (i = 0; i < sockets; i++)
 	    if (socket_table[i] == NULL) break;
@@ -339,7 +335,7 @@ static void shutdown_socket(u_long i)
 #endif
 
     /* Blank out the socket state */
-    s->state &= SOCKET_PRESENT|SOCKET_RESET_PENDING|SOCKET_SETUP_PENDING;
+    s->state &= SOCKET_PRESENT|SOCKET_SETUP_PENDING;
     init_socket(s);
     s->irq.AssignedIRQ = s->irq.Config = 0;
     s->functions = 0;
@@ -403,10 +399,14 @@ static void reset_socket(u_long i)
     s->socket.flags &= ~SS_RESET;
     s->ss_entry(s->sock, SS_SetSocket, &s->socket);
     s->unreset_timeout = 0;
-    s->unreset.expires = RUN_AT(unreset_check);
-    add_timer(&s->unreset);
+    s->setup.expires = RUN_AT(unreset_delay);
+    s->setup.function = &unreset_socket;
+    add_timer(&s->setup);
 } /* reset_socket */
 
+#define EVENT_MASK \
+(SOCKET_SETUP_PENDING|SOCKET_SUSPEND|SOCKET_RESET_PENDING)
+
 static void unreset_socket(u_long i)
 {
     socket_info_t *s = socket_table[i];
@@ -419,23 +419,21 @@ static void unreset_socket(u_long i)
 	if (pc_debug)
 	    printk(KERN_DEBUG "cs: reset done on socket %ld\n", i);
 #endif
-	if (s->state & SOCKET_SETUP_PENDING) {
-	    send_event(s, CS_EVENT_CARD_INSERTION, CS_EVENT_PRI_LOW);
-	    s->state &= ~SOCKET_SETUP_PENDING;
-	}
-	else if (s->state & SOCKET_SUSPEND) {
+	if (s->state & SOCKET_SUSPEND) {
+	    s->state &= ~EVENT_MASK;
 	    if (verify_cis_cache(s) != 0)
 		parse_events(s, SS_DETECT);
 	    else
 		send_event(s, CS_EVENT_PM_RESUME, CS_EVENT_PRI_LOW);
-	    s->state &= ~SOCKET_SUSPEND;
-	}
-	else {
+	} else if (s->state & SOCKET_SETUP_PENDING) {
+	    send_event(s, CS_EVENT_CARD_INSERTION, CS_EVENT_PRI_LOW);
+	    s->state &= ~SOCKET_SETUP_PENDING;
+	} else {
 	    send_event(s, CS_EVENT_CARD_RESET, CS_EVENT_PRI_LOW);
 	    s->reset_handle->event_callback_args.info = NULL;
 	    EVENT(s->reset_handle, CS_EVENT_RESET_COMPLETE,
 		  CS_EVENT_PRI_LOW);
-	    s->state &= ~SOCKET_RESET_PENDING;
+	    s->state &= ~EVENT_MASK;
 	}
     }
     else {
@@ -446,12 +444,12 @@ static void unreset_socket(u_long i)
 	if (s->unreset_timeout > unreset_limit) {
 	    printk(KERN_NOTICE "cs: socket %ld timed out during"
 		   " reset\n", i);
-	    s->state &= ~SOCKET_RESET_PENDING;
+	    s->state &= ~EVENT_MASK;
 	}
 	else {
 	    s->unreset_timeout++;
-	    s->unreset.expires = RUN_AT(unreset_check);
-	    add_timer(&s->unreset);
+	    s->setup.expires = RUN_AT(unreset_check);
+	    add_timer(&s->setup);
 	}
     }
 } /* unreset_socket */
@@ -495,12 +493,12 @@ static void do_shutdown(socket_info_t *s)
     for (client = s->clients; client; client = client->next)
 	if (!(client->Attributes & INFO_MASTER_CLIENT))
 	    client->state |= CLIENT_STALE;
-    if (s->state & SOCKET_SETUP_PENDING) {
+    if (s->state & (SOCKET_SETUP_PENDING|SOCKET_RESET_PENDING)) {
 #ifdef PCMCIA_DEBUG
 	printk(KERN_DEBUG "cs: flushing pending setup\n");
 #endif
 	del_timer(&s->setup);
-	s->state &= ~SOCKET_SETUP_PENDING;
+	s->state &= ~EVENT_MASK;
     }
     s->shutdown.expires = RUN_AT(shutdown_delay);
     add_timer(&s->shutdown);
@@ -514,7 +512,7 @@ static void parse_events(void *info, u_long events)
 	int status;
 	cli();
 	if ((s->state & SOCKET_PRESENT) &&
-	    !(s->state & SOCKET_SHUTDOWN_PENDING))
+	    !(s->state & (SOCKET_SUSPEND|SOCKET_SHUTDOWN_PENDING)))
 	    do_shutdown(s);
 	s->ss_entry(s->sock, SS_GetStatus, &status);
 	if (status & SS_DETECT) {
@@ -528,7 +526,10 @@ static void parse_events(void *info, u_long events)
 		del_timer(&s->setup);
 	    }
 	    s->setup.function = &setup_socket;
-	    s->setup.expires = RUN_AT(setup_delay);
+	    if (s->state & SOCKET_SUSPEND)
+		s->setup.expires = RUN_AT(resume_delay);
+	    else
+		s->setup.expires = RUN_AT(setup_delay);
 	    add_timer(&s->setup);
 	}
 	sti();
@@ -601,20 +602,10 @@ static int handle_apm_event(apm_event_t event)
 	    /* Do this just to reinitialize the socket */
 	    s->ss_entry(s->sock, SS_SetSocket, &dead_socket);
 	    s->ss_entry(s->sock, SS_GetStatus, &stat);
-	    /* Did we miss an event? */
-	    if (s->state & SOCKET_PRESENT) {
-		if (stat & SS_DETECT)
-		    /* Normal resume -- assume card is the same */
-		    setup_socket(i);
-		else
-		    /* We missed a removal event */
-		    parse_events(s, SS_DETECT);
-	    } else {
-		if (stat & SS_DETECT)
-		    /* we missed an insertion event */
-		    parse_events(s, SS_DETECT);
-		/* Otherwise, nothing to do */
-	    }
+	    /* If there was or is a card here, we need to do something
+	       about it... but parse_events will sort it all out. */
+       	    if ((s->state & SOCKET_PRESENT) || (stat & SS_DETECT))
+		parse_events(s, SS_DETECT);
 	}
 	break;
     }
@@ -775,6 +766,7 @@ static int bind_device(bind_req_t *req)
     client->state = CLIENT_UNBOUND;
     client->erase_busy.next = &client->erase_busy;
     client->erase_busy.prev = &client->erase_busy;
+    init_waitqueue(&client->mtd_req);
     client->next = s->clients;
     s->clients = client;
 #ifdef PCMCIA_DEBUG
@@ -2002,7 +1994,7 @@ int CardServices(int func, void *a1, void *a2, void *a3)
 
 /*====================================================================*/
 
-int init_cs(void)
+int init_module(void)
 {
     printk(KERN_INFO "%s\n", release);
 #ifdef PCMCIA_DEBUG
@@ -2016,7 +2008,6 @@ int init_cs(void)
     return 0;
 }
 
-#ifdef MODULE
 void cleanup_module(void)
 {
     printk(KERN_INFO "unloading PCMCIA Card Services\n");
@@ -2025,4 +2016,3 @@ void cleanup_module(void)
 #endif
     release_resource_db();
 }
-#endif /* MODULE */
