@@ -6,12 +6,8 @@
     
 ======================================================================*/
 
-#include "config.h"
-#include "k_compat.h"
-
-#ifdef MODULE
-#define init_ds init_module
-#endif
+#include <pcmcia/config.h>
+#include <pcmcia/k_compat.h>
 
 #include <linux/kernel.h>
 #include <linux/major.h>
@@ -26,17 +22,17 @@
 
 #include <asm/segment.h>
 
-#include "version.h"
-#include "cs_types.h"
-#include "cs.h"
-#include "bulkmem.h"
-#include "cistpl.h"
-#include "ds.h"
+#include <pcmcia/version.h>
+#include <pcmcia/cs_types.h>
+#include <pcmcia/cs.h>
+#include <pcmcia/bulkmem.h>
+#include <pcmcia/cistpl.h>
+#include <pcmcia/ds.h>
 
 #ifdef PCMCIA_DEBUG
 int pc_debug = PCMCIA_DEBUG;
 static const char *version =
-    "ds.c 1.57 1996/09/08 22:07:27 (David Hinds)";
+"ds.c 1.63 1996/12/21 21:34:41 (David Hinds)";
 #endif
 
 /*====================================================================*/
@@ -45,10 +41,10 @@ static void handle_removal(u_long i);
 
 static int ds_event(event_t event, int priority,
 		    event_callback_args_t *args);
-static int ds_read(struct inode *inode, struct file *file,
-		   char *buf, int count);
-static int ds_write(struct inode *inode, struct file *file,
-		    CONST char *buf, int count);
+static FS_SIZE_T ds_read(struct inode *inode, struct file *file,
+			 char *buf, U_FS_SIZE_T count);
+static FS_SIZE_T ds_write(struct inode *inode, struct file *file,
+			  CONST char *buf, U_FS_SIZE_T count);
 static int ds_select(struct inode *inode, struct file *file,
 		     int sel_type, select_table *wait);
 static int ds_ioctl(struct inode * inode, struct file * file,
@@ -130,6 +126,8 @@ static struct symbol_table ds_symtab = {
 #endif
     X(register_pcmcia_driver),
     X(unregister_pcmcia_driver),
+    X(bind_request),
+    X(bind_mtd),
 #include <linux/symtab_end.h>
 };
 
@@ -158,6 +156,11 @@ int register_pcmcia_driver(dev_info_t *dev_info,
 	printk(KERN_DEBUG "ds: register_pcmcia_driver('%s')\n",
 	       (char *)dev_info);
 #endif
+    for (driver = root_driver; driver; driver = driver->next)
+	if (strcmp((char *)dev_info, (char *)driver->dev_info) == 0)
+	    break;
+    if (driver)
+	return -1;
     MOD_INC_USE_COUNT;
     driver = kmalloc(sizeof(driver_info_t), GFP_KERNEL);
     driver->next = root_driver;
@@ -308,7 +311,7 @@ static int ds_event(event_t event, int priority,
     
 ======================================================================*/
 
-static int bind_mtd(int i, mtd_info_t *mtd_info)
+int bind_mtd(int i, mtd_info_t *mtd_info)
 {
     struct driver_info_t *driver;
     mtd_bind_t bind_req;
@@ -345,7 +348,7 @@ static int bind_mtd(int i, mtd_info_t *mtd_info)
     
 ======================================================================*/
 
-static int bind_request(int i, bind_info_t *bind_info)
+int bind_request(int i, bind_info_t *bind_info)
 {
     struct driver_info_t *driver;
     socket_bind_t *b;
@@ -554,13 +557,12 @@ static void ds_release(struct inode *inode, struct file *file)
 
 /*====================================================================*/
 
-static int ds_read(struct inode *inode, struct file *file,
-		   char *buf, int count)
+static FS_SIZE_T ds_read(struct inode *inode, struct file *file,
+			 char *buf, U_FS_SIZE_T count)
 {
     socket_t i = MINOR(inode->i_rdev);
     socket_info_t *s;
     user_info_t *user;
-    int val;
 
 #ifdef PCMCIA_DEBUG
     if (pc_debug)
@@ -581,15 +583,14 @@ static int ds_read(struct inode *inode, struct file *file,
 	if (current->signal & ~current->blocked)
 	    return -EINTR;
     }
-    val = get_queued_event(user);
-    put_fs_long(val, buf);
+    put_user(get_queued_event(user), (long *)buf);
     return 4;
 } /* ds_read */
 
 /*====================================================================*/
 
-static int ds_write(struct inode *inode, struct file *file,
-		    CONST char *buf, int count)
+static FS_SIZE_T ds_write(struct inode *inode, struct file *file,
+			  CONST char *buf, U_FS_SIZE_T count)
 {
     socket_t i = MINOR(inode->i_rdev);
     socket_info_t *s;
@@ -613,7 +614,7 @@ static int ds_write(struct inode *inode, struct file *file,
 
     if (s->req_pending) {
 	s->req_pending--;
-	s->req_result = get_fs_long(buf);
+	get_user(s->req_result, (long *)buf);
 	if ((s->req_result != 0) || (s->req_pending == 0))
 	    wake_up_interruptible(&s->request);
     }
@@ -702,7 +703,7 @@ static int ds_ioctl(struct inode * inode, struct file * file,
     
     err = ret = 0;
     
-    if (cmd & IOC_IN) memcpy_fromfs((char *)&buf, (char *)arg, size);
+    if (cmd & IOC_IN) copy_from_user((char *)&buf, (char *)arg, size);
     
     switch (cmd) {
     case DS_ADJUST_RESOURCE_INFO:
@@ -808,14 +809,14 @@ static int ds_ioctl(struct inode * inode, struct file * file,
 	}
     }
     
-    if (cmd & IOC_OUT) memcpy_tofs((char *)arg, (char *)&buf, size);
+    if (cmd & IOC_OUT) copy_to_user((char *)arg, (char *)&buf, size);
      
     return err;
 } /* ds_ioctl */
 
 /*====================================================================*/
 
-int init_ds(void)
+int init_module(void)
 {
     client_reg_t client_reg;
     servinfo_t serv;
@@ -840,7 +841,8 @@ int init_ds(void)
 	s->state = 0;
 	s->user = NULL;
 	s->req_pending = 0;
-	s->queue = s->request = NULL;
+	init_waitqueue(&s->queue);
+	init_waitqueue(&s->request);
 	s->handle = NULL;
 	s->removal.prev = s->removal.next = NULL;
 	s->removal.data = i;
@@ -887,7 +889,6 @@ int init_ds(void)
     return 0;
 }
 
-#ifdef MODULE
 void cleanup_module(void)
 {
     int i;
@@ -898,4 +899,3 @@ void cleanup_module(void)
     sockets = 0;
     kfree(socket_table);
 }
-#endif /* MODULE */
