@@ -15,12 +15,8 @@
 
 ======================================================================*/
 
-#include "config.h"
-#include "k_compat.h"
-
-#ifdef MODULE
-#define init_fmvj18x_cs init_module
-#endif
+#include <pcmcia/config.h>
+#include <pcmcia/k_compat.h>
 
 #include <linux/kernel.h>
 #include <linux/sched.h>
@@ -40,11 +36,11 @@
 #include <linux/if_arp.h>
 #include <linux/ioport.h>
 
-#include "version.h"
-#include "cs_types.h"
-#include "cs.h"
-#include "cistpl.h"
-#include "ds.h"
+#include <pcmcia/version.h>
+#include <pcmcia/cs_types.h>
+#include <pcmcia/cs.h>
+#include <pcmcia/cistpl.h>
+#include <pcmcia/ds.h>
 
 /*
    All the PCMCIA modules use PCMCIA_DEBUG to control debugging.  If
@@ -655,7 +651,7 @@ static int fmvj18x_init(struct device *dev)
 
 /*====================================================================*/
 
-int init_fmvj18x_cs(void)
+int init_module(void)
 {
     servinfo_t serv;
 #ifdef PCMCIA_DEBUG
@@ -674,7 +670,6 @@ int init_fmvj18x_cs(void)
 
 /*====================================================================*/
 
-#ifdef MODULE
 void cleanup_module(void)
 {
 #ifdef PCMCIA_DEBUG
@@ -688,7 +683,6 @@ void cleanup_module(void)
 	fmvj18x_detach(dev_list);
     }
 }
-#endif /* MODULE */
 
 /*====================================================================*/
 
