@@ -9,15 +9,11 @@
     
 ======================================================================*/
 
-#include "config.h"
-#include "k_compat.h"
+#include <pcmcia/config.h>
+#include <pcmcia/k_compat.h>
 
 /* #define PSYCHO_DEBUG */
 
-#ifdef MODULE
-#define init_ftl_cs init_module
-#endif
-
 #include <linux/kernel.h>
 #include <linux/sched.h>
 #include <linux/ptrace.h>
@@ -32,13 +28,17 @@
 #include <asm/segment.h>
 #include <stdarg.h>
 
-#include "version.h"
-#include "cs_types.h"
-#include "cs.h"
-#include "bulkmem.h"
-#include "cistpl.h"
-#include "ds.h"
-#include "ftl.h"
+#if (LINUX_VERSION_CODE >= VERSION(2,1,0))
+#include <linux/vmalloc.h>
+#endif
+
+#include <pcmcia/version.h>
+#include <pcmcia/cs_types.h>
+#include <pcmcia/cs.h>
+#include <pcmcia/bulkmem.h>
+#include <pcmcia/cistpl.h>
+#include <pcmcia/ds.h>
+#include <pcmcia/ftl.h>
 
 /* Major device # for FTL device */
 static int major_dev = 0;
@@ -59,7 +59,7 @@ static int major_dev = 0;
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
 static char *version =
-"ftl_cs.c 1.20 1996/09/21 05:07:21 (David Hinds)";
+"ftl_cs.c 1.25 1996/12/21 19:30:48 (David Hinds)";
 #endif
 
 static int shuffle_freq = 50;
@@ -201,6 +201,7 @@ static dev_link_t *ftl_attach(void)
     
     dev = kmalloc(sizeof(struct ftl_dev_t), GFP_KERNEL);
     memset(dev, 0, sizeof(ftl_dev_t));
+    init_waitqueue(&dev->erase_pending);
     link->priv = dev;
 
     /* Register with Card Services */
@@ -1034,8 +1035,10 @@ static int ftl_open(struct inode *inode, struct file *file)
 	open.Attributes = partition->region.Attributes;
 	open.Offset = partition->region.CardOffset;
 	ret = CardServices(OpenMemory, &partition->handle, &open);
-	if (ret != CS_SUCCESS)
+	if (ret != CS_SUCCESS) {
+	    cs_error(OpenMemory, ret);
 	    return -ENODEV;
+	}
 	if (scan_partition(partition) == 0) {
 	    partition->state = FTL_FORMATTED;
 #ifdef PCMCIA_DEBUG
@@ -1348,8 +1351,8 @@ static int ftl_ioctl(struct inode *inode, struct file *file,
 	else {
 	    ret = verify_area(VERIFY_WRITE, (long *)arg, sizeof(long));
 	    if (ret) break;
-	    put_fs_long(partition->header.FormattedSize/SECTOR_SIZE,
-			(long *)arg);
+	    put_user(partition->header.FormattedSize/SECTOR_SIZE,
+		     (long *)arg);
 	}
 	break;
     default:
@@ -1417,7 +1420,7 @@ static void do_ftl_request(void)
 
 /*====================================================================*/
 
-int init_ftl_cs(void)
+int init_module(void)
 {
     servinfo_t serv;
     int i;
@@ -1449,7 +1452,6 @@ int init_ftl_cs(void)
     return 0;
 }
 
-#ifdef MODULE
 void cleanup_module(void)
 {
     int i;
@@ -1471,5 +1473,3 @@ void cleanup_module(void)
 	}
     }
 }
-#endif /* MODULE */
-
