@@ -6,12 +6,8 @@
     
 ======================================================================*/
 
-#include "config.h"
-#include "k_compat.h"
-
-#ifdef MODULE
-#define init_i82365 init_module
-#endif
+#include <pcmcia/config.h>
+#include <pcmcia/k_compat.h>
 
 #include <sys/types.h>
 #include <linux/fcntl.h>
@@ -31,19 +27,29 @@
 #include <linux/bios32.h>
 #include <linux/ioport.h>
 
-#include "version.h"
-#include "ss.h"
+#include <pcmcia/version.h>
+#include <pcmcia/ss.h>
+
+/* ISA-to-PCMCIA controllers */
 #include "i82365.h"
 #include "pd67xx.h"
 #include "vg468.h"
 #include "ricoh.h"
-#include "cs_types.h"
-#include "cs.h"
+
+/* PCI-to-CardBus controllers */
+#include "yenta.h"
+#include "rl5c466.h"
+#include "ti113x.h"
+#include "smc34c90.h"
+#include "pd6832.h"
+
+#include <pcmcia/cs_types.h>
+#include <pcmcia/cs.h>
 
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
 static const char *version =
-"i82365.c 1.102 1996/10/01 14:25:39 (David Hinds)";
+"i82365.c 1.104 1996/12/21 19:30:49 (David Hinds)";
 #endif
 
 /*====================================================================*/
@@ -120,9 +126,13 @@ typedef struct socket_info_t {
 #define IS_PD6710	0x40
 #define IS_PD672X	0x41
 #define IS_PD6729	0x42
+#define IS_PD6832	0x43
 #define IS_VG468	0x50
 #define IS_VG469	0x51
-#define IS_RICOH	0x60
+#define IS_RF5Cx96	0x60
+#define IS_RL5C466	0x61
+#define IS_SMC34C90	0x70
+#define IS_TI113X	0x80
 
 /* Where we keep track of our sockets... */
 static int sockets = 0;
@@ -304,7 +314,7 @@ static int identify(int sock)
     /* Check for Ricoh chips */
     val = i365_get(sock, RF5C_CHIP_ID);
     if ((val == RF5C_CHIP_RF5C296) || (val == RF5C_CHIP_RF5C396))
-	type = IS_RICOH;
+	type = IS_RF5Cx96;
     
     /* Check for Cirrus CL-PD67xx chips */
     i365_set(sock, PD67_CHIP_INFO, 0);
@@ -319,6 +329,13 @@ static int identify(int sock)
 
 /*====================================================================*/
 
+static void add_socket(int type, int psock)
+{
+    socket_table[sockets].psock = psock;
+    socket_table[sockets].type = type;
+    sockets++;
+}
+
 static void add_pcic(int type, int ns)
 {
     int i, j, irq;
@@ -345,14 +362,22 @@ static void add_pcic(int type, int ns)
 	break;
     case IS_PD6729:
 	printk("Cirrus PD6729 PCI"); break;
+    case IS_PD6832:
+	printk("Cirrus PD6832 CardBus"); break;
     case IS_IBM:
 	printk("IBM Clone"); break;
     case IS_VG468:
 	printk("Vadem VG-468"); break;
     case IS_VG469:
 	printk("Vadem VG-469"); break;
-    case IS_RICOH:
+    case IS_RF5Cx96:
 	printk("Ricoh RF5C296/396"); break;
+    case IS_RL5C466:
+	printk("Ricoh RL5C466 CardBus"); break;
+    case IS_SMC34C90:
+	printk("SMC 34C90 CardBus"); break;
+    case IS_TI113X:
+	printk("TI 113X CardBus"); break;
     }
     base = sockets-ns;
     printk(" at %#x", I365_PORT(socket_table[base].psock));
@@ -381,16 +406,20 @@ static void add_pcic(int type, int ns)
     }
     printk(") = 0x%lx, ", mask);
     
-    /* Check for only two interrupts available */
-    scan = (mask & (mask-1));
-    if (((scan & (scan-1)) == 0) && (poll_interval == 0))
-	poll_interval = HZ;
+    /* Poll if only two interrupts available, or if we may have more
+       than one socket controller. */
+    if (poll_interval == 0) {
+	scan = (mask & (mask-1));
+	if (extra_sockets || ((scan & (scan-1)) == 0))
+	    poll_interval = HZ;
+    }
 
     /* Pick a card status change interrupt */
+    irq = 0;
     if (poll_interval == 0) {
 	if (cs_irq == 0) {
 	    for (i = 15; i > 0; i--)
-		if ((mask & (1 << i)) &&
+		if ((i != 12) && (mask & (1 << i)) &&
 		    (REQUEST_IRQ(i, i365_interrupt, 0, "i82365") == 0))
 		    break;
 	    irq = i;
@@ -398,13 +427,9 @@ static void add_pcic(int type, int ns)
 	else {
 	    if (REQUEST_IRQ(cs_irq, i365_interrupt, 0, "i82365") == 0)
 		irq = cs_irq;
-	    else
-		irq = 0;
 	}
-	if (irq == 0) poll_interval = 100;
+	if (irq == 0) poll_interval = HZ;
     }
-    else
-	irq = 0;
     
     if (irq)
 	printk("status change on irq %d\n", irq);
@@ -454,42 +479,103 @@ static int is_active(int s)
 
 /*====================================================================*/
 
-static int i365_init(void)
+#ifdef CONFIG_PCI
+
+typedef struct {
+    u_short	vendor;
+    u_short	device;
+    u_short	type;
+} pci_id_t;
+
+static pci_id_t pci_id[] = {
+    { PCI_VENDOR_ID_RICOH, PCI_DEVICE_ID_RICOH_RL5C466, IS_RL5C466 },
+    { PCI_VENDOR_ID_CIRRUS, PCI_DEVICE_ID_CIRRUS_6832, IS_PD6832 },
+    { PCI_VENDOR_ID_SMC, PCI_DEVICE_ID_SMC_34C90, IS_SMC34C90 },
+    { PCI_VENDOR_ID_TI, PCI_DEVICE_ID_TI_1131, IS_TI113X }
+};
+#define PCI_COUNT (sizeof(pci_id)/sizeof(pci_id_t))
+
+/* Default settings for PCI command configuration register */
+#define CMD_DFLT (PCI_COMMAND_IO|PCI_COMMAND_MEMORY| \
+		  PCI_COMMAND_MASTER|PCI_COMMAND_WAIT)
+
+static void pci_setup(int type, u_char bus, u_char fn)
+{
+    u_short s;
+
+    pcibios_write_config_word(bus, fn, PCI_COMMAND, CMD_DFLT);
+    switch (type) {
+    case IS_TI113X:
+	pcibios_read_config_word(bus, fn, TI113X_DCR, &s);
+	s = (s & ~TI113X_DCR_IMODE_MASK) | TI113X_DCR_IMODE_ISA;
+	pcibios_write_config_word(bus, fn, TI113X_DCR, s);
+	break;
+    case IS_RL5C466:
+	s = RL5C466_16CTL_LEVEL_1;
+	pcibios_write_config_word(bus, fn, RL5C466_16BIT_CTL, s);
+	break;
+    case IS_PD6832:
+	pcibios_read_config_word(bus, fn, CB_BRIDGE_CONTROL, &s);
+	s |= PD6832_BCR_MGMT_IRQ_ENA;
+	pcibios_write_config_word(bus, fn, CB_BRIDGE_CONTROL, s);
+	break;
+    }
+    /* Enable ISA interrupt steering */
+    pcibios_read_config_word(bus, fn, CB_BRIDGE_CONTROL, &s);
+    s |= CB_BCR_ISA_IRQ;
+    pcibios_write_config_word(bus, fn, CB_BRIDGE_CONTROL, s);
+    /* Enable 16-bit legacy mode */
+    pcibios_write_config_dword(bus, fn, CB_LEGACY_MODE_BASE,
+			       i365_base | 0x01);
+}
+
+static void pci_probe(void)
+{
+    int i, index;
+    u_char bus, fn;
+    u_int addr;
+    
+    /* Check for Cirrus PD6729 PCI controller */
+    if (pcibios_find_device(PCI_VENDOR_ID_CIRRUS,
+			    PCI_DEVICE_ID_CIRRUS_6729,
+			    0, &bus, &fn) == 0) {
+	pcibios_read_config_dword(bus, fn, PCI_BASE_ADDRESS_0, &addr);
+	i365_base = addr & ~0x1;
+	pcibios_write_config_word(bus, fn, PCI_COMMAND, CMD_DFLT);
+	add_socket(IS_PD6729, 0);
+	add_socket(IS_PD6729, 1);
+	add_pcic(IS_PD6729, 2);
+    }
+
+    /* Check for CardBus controllers */
+    for (i = 0; i < PCI_COUNT; i++) {
+	for (index = 0; index < 2; index++) {
+	    if (pcibios_find_device(pci_id[i].vendor,
+				    pci_id[i].device,
+				    index, &bus, &fn) != 0)
+		break;
+	    pci_setup(pci_id[i].type, bus, fn);
+	    add_socket(pci_id[i].type, index);
+	}
+	if (index != 0)
+	    add_pcic(pci_id[i].type, index);
+    }
+
+}
+#endif /* CONFIG_PCI */
+
+/*====================================================================*/
+
+static void isa_probe(void)
 {
     int i, j, k, p, ns;
-    int id, type, is_pci = 0;
-
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_INFO "%s\n", version);
-#endif
+    int id, type;
     
-    printk(KERN_INFO "Intel PCIC probe: ");
-    
-#ifdef CONFIG_PCI
-    if (pcibios_present()) {
-	u_char bus, device_fn;
-	/* Check for Cirrus PD6729 PCI controller */
-	i = pcibios_find_device(PCI_VENDOR_ID_CIRRUS,
-				PCI_DEVICE_ID_CIRRUS_6729,
-				0, &bus, &device_fn);
-	if (i == 0) {
-	    unsigned long addr;
-	    pcibios_read_config_dword(bus, device_fn, 0x10, (void *)&addr);
-	    i365_base = addr & ~0x1;
-	    pcibios_write_config_dword(bus, device_fn, 0x4, 0x00ff00ff);
-	    is_pci = 1;
-	}
-    }
-#endif
-
     if (check_region(i365_base, 8) != 0) {
 	printk("port conflict at %#x\n", i365_base);
-	return -ENODEV;
+	return;
     }
 
-    sockets = 0;
-
     /* Wake up a sleepy Cirrus controller */
     if (wakeup) {
 	i365_bclr(0, PD67_MISC_CTL_2, PD67_MC2_SUSPEND);
@@ -502,23 +588,15 @@ static int i365_init(void)
 	for (i = 0; i < 4; i++) {
 	    if (i == ignore) continue;
 	    j = ((i & 1) << 3) + (i << 1);
-	    if ((identify(j) == IS_I82365DF) && !is_active(j)) {
-		socket_table[sockets].psock = j;
-		socket_table[sockets].type = IS_VLSI;
-		sockets++;
-	    }
+	    if ((identify(j) == IS_I82365DF) && !is_active(j))
+		add_socket(IS_VLSI, j);
 	}
 	add_pcic(IS_VLSI, sockets);
     } else {
 	for (i = 0; i < (extra_sockets ? 8 : 4); i += 2) {
 	    id = identify(i);
 	    if (id < 0) continue;
-	    
-	    if (is_pci && (id == IS_PD672X))
-		type = IS_PD6729;
-	    else
-		type = id;
-	    
+
 	    ns = 0;
 	    for (j = 0; j < 2; j++) {
 		/* Does the socket exist? */
@@ -537,17 +615,36 @@ static int i365_init(void)
 		if (i365_get(i+j, I365_MEM(0)+I365_W_OFF) != i+j)
 		    break;
 		if (k == sockets) {
-		    socket_table[sockets].psock = i+j;
-		    socket_table[sockets].type = type;
-		    sockets++;
+		    add_socket(type, i+j);
 		    ns++;
 		}
 	    }
-	    /* Some probes are more reliable than others... */
 	    if (ns != 0)
 		add_pcic(type, ns);
 	}
     }
+}
+
+/*====================================================================*/
+
+static int i365_init(void)
+{
+    int i;
+    
+#ifdef PCMCIA_DEBUG
+    if (pc_debug)
+	printk(KERN_INFO "%s\n", version);
+#endif
+    printk(KERN_INFO "Intel PCIC probe: ");
+    sockets = 0;
+
+#ifdef CONFIG_PCI
+    if (pcibios_present())
+	pci_probe();
+#endif
+
+    if (sockets == 0)
+	isa_probe();
 	
     if (sockets == 0) {
 	printk("not found.\n");
@@ -1168,7 +1265,7 @@ static int i365_service(u_int lsock, u_int cmd, void *arg)
 
 /*====================================================================*/
 
-int init_i82365(void)
+int init_module(void)
 {
     servinfo_t serv;
     CardServices(GetCardServicesInfo, &serv);
@@ -1180,9 +1277,7 @@ int init_i82365(void)
     return i365_init();
 }
 
-#ifdef MODULE
 void cleanup_module(void)
 {
     i365_finish();
 }
-#endif /* MODULE */
