@@ -12,12 +12,8 @@
 
 ======================================================================*/
 
-#include "config.h"
-#include "k_compat.h"
-
-#ifdef MODULE
-#define init_ibmtr_cs init_module
-#endif
+#include <pcmcia/config.h>
+#include <pcmcia/k_compat.h>
 
 #include <linux/kernel.h>
 #include <linux/sched.h>
@@ -33,11 +29,11 @@
 #include <linux/trdevice.h>
 #include <drivers/net/ibmtr.h>
 
-#include "version.h"
-#include "cs_types.h"
-#include "cs.h"
-#include "cistpl.h"
-#include "ds.h"
+#include <pcmcia/version.h>
+#include <pcmcia/cs_types.h>
+#include <pcmcia/cs.h>
+#include <pcmcia/cistpl.h>
+#include <pcmcia/ds.h>
 
 extern struct device *irq2dev_map[16];
 
@@ -570,7 +566,7 @@ static void ibmtr_hw_setup(struct device *dev)
 }
 
 
-int init_ibmtr_cs(void)
+int init_module(void)
 {
     servinfo_t serv;
 #ifdef PCMCIA_DEBUG
@@ -587,7 +583,6 @@ int init_ibmtr_cs(void)
     return 0;
 }
 
-#ifdef MODULE
 void cleanup_module(void)
 {
 #ifdef PCMCIA_DEBUG
@@ -598,5 +593,3 @@ void cleanup_module(void)
     while (dev_list != NULL)
         ibmtr_detach(dev_list);
 }
-#endif /* MODULE */
-
