@@ -11,15 +11,11 @@
     
 ======================================================================*/
 
-#include "config.h"
-#include "k_compat.h"
+#include <pcmcia/config.h>
+#include <pcmcia/k_compat.h>
 
 /* #define PCMCIA_DEBUG 1 */
 
-#ifdef MODULE
-#define init_iflash2X_mtd init_module
-#endif
-
 #include <linux/kernel.h>
 #include <linux/sched.h>
 #include <linux/ptrace.h>
@@ -34,25 +30,26 @@
 #include <asm/segment.h>
 #include <stdarg.h>
 
-#include "version.h"
-#include "cs_types.h"
-#include "cs.h"
-#include "bulkmem.h"
-#include "cistpl.h"
-#include "ds.h"
-#include "mem_op.h"
+#include <pcmcia/version.h>
+#include <pcmcia/cs_types.h>
+#include <pcmcia/cs.h>
+#include <pcmcia/bulkmem.h>
+#include <pcmcia/cistpl.h>
+#include <pcmcia/ds.h>
+#include <pcmcia/mem_op.h>
 #include "iflash.h"
 
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
 static char *version =
-"iflash2+_mtd.c 1.23 1996/05/28 15:14:24 (David Hinds)";
+"iflash2+_mtd.c 1.29 1996/12/21 19:30:50 (David Hinds)";
 #endif
 
 /*====================================================================*/
 
 static int vpp_timeout_period	= HZ;		/* in ticks */
 static int vpp_settle		= HZ/10;	/* in ticks */
+static int write_timeout	= HZ/10;	/* in ticks */
 static int erase_timeout	= 100;		/* in ms */
 static int erase_limit		= 10*HZ;	/* in ticks */
 
@@ -95,7 +92,6 @@ typedef struct flash_dev_t {
     int			vpp_usage;
     u_long		vpp_start;
     struct timer_list	vpp_timeout;
-    struct wait_queue	write_queue;
     flash_region_t	*flash[2*CISTPL_MAX_DEVICES];
 } flash_dev_t;
 
@@ -143,8 +139,7 @@ static void log_esr(char *s, volatile u_short *esr)
     writew(IF_READ_ESR, esr);
     BSR = readw(esr+2);
     GSR = readw(esr+4);
-    if (s)
-	printk("%s: ", s);
+    printk(KERN_NOTICE "%s%s", s ? s : "", s ? ": " : "");
     printk("CSR = 0x%04x, BSR = 0x%04x, GSR = 0x%04x\n",
 	   CSR, BSR, GSR);
 }
@@ -195,8 +190,10 @@ static int check_write(struct wait_queue **queue, volatile u_short *esr)
 {
     writew(IF_READ_ESR, esr);
     if ((readw(esr+2) & BSR_READY) != BSR_READY) {
-	current->timeout = jiffies + 2;
-	interruptible_sleep_on(queue);
+	current->timeout = jiffies + write_timeout;
+	while (current->timeout &&
+	       ((readw(esr+2) & BSR_READY) != BSR_READY))
+	    interruptible_sleep_on(queue);
 	if ((readw(esr+2) & BSR_READY) != BSR_READY) {
 	    printk(KERN_NOTICE "iflash2+_mtd: check_write: timed out!\n");
 	    log_esr(NULL, esr);
@@ -218,8 +215,10 @@ static int page_setup(struct wait_queue **queue, volatile u_short *esr,
 
     writew(IF_READ_ESR, esr);
     if ((readw(esr+4) & GSR_PAGE_AVAIL) != GSR_PAGE_AVAIL) {
-	current->timeout = jiffies + 2;
-	interruptible_sleep_on(queue);
+	current->timeout = jiffies + write_timeout;
+	while (current->timeout &&
+	       ((readw(esr+4) & GSR_PAGE_AVAIL) != GSR_PAGE_AVAIL))
+	    interruptible_sleep_on(queue);
 	if ((readw(esr+4) & GSR_PAGE_AVAIL) != GSR_PAGE_AVAIL) {
 	    printk(KERN_NOTICE "iflash2+_mtd: page_setup timed out\n");
 	    log_esr(NULL, esr);
@@ -457,6 +456,7 @@ static dev_link_t *flash_attach(void)
     memset(link, 0, sizeof(*link));
     link->release.function = &flash_release;
     link->release.data = (u_long)link;
+    init_waitqueue(&link->pending);
     dev = kmalloc(sizeof(struct flash_dev_t), GFP_KERNEL);
     memset(dev, 0, sizeof(*dev));
     dev->vpp_timeout.function = vpp_off;
@@ -748,9 +748,9 @@ static int flash_read(dev_link_t *link, char *buf, mtd_request_t *req)
 	nb = (from+length > WINDOW_SIZE) ? WINDOW_SIZE-from : length;
 
 	if (req->Function & MTD_REQ_KERNEL)
-	    memcpy_frompc(buf, &dev->Base[from], nb);
+	    copy_from_pc(buf, &dev->Base[from], nb);
 	else
-	    memcpy_frompc_tofs(buf, &dev->Base[from], nb);
+	    copy_pc_to_user(buf, &dev->Base[from], nb);
 	
 	buf += nb;
 	from = 0;
@@ -790,7 +790,8 @@ static int basic_write(struct wait_queue **queue, char *esr, char *dest,
     int ret;
 
     /* Enable interrupts on write complete */
-    set_rdy_mode((u_short *)esr, IF_RDY_PULSE_WRITE);
+    ret = set_rdy_mode((u_short *)esr, IF_RDY_PULSE_WRITE);
+    if (ret != CS_SUCCESS) return ret;
     
     /* Fix for mis-aligned writes */
     if ((u_long)dest & 1) {
@@ -801,7 +802,13 @@ static int basic_write(struct wait_queue **queue, char *esr, char *dest,
 #endif
 	ret = page_setup(queue, (u_short *)esr, (u_short *)dest, 1);
 	if (ret != CS_SUCCESS) return ret;
-	writeb((is_krnl) ? (*buf) : get_fs_byte(buf), dest);
+	if (is_krnl)
+	    writeb(*buf, dest);
+	else {
+	    char c;
+	    get_user(c, buf);
+	    writeb(c, dest);
+	}
 	ret = page_write((u_short *)esr, (u_short *)dest, 1);
 	if (ret != CS_SUCCESS) return ret;
 	dest++; buf++; nb--;
@@ -816,9 +823,9 @@ static int basic_write(struct wait_queue **queue, char *esr, char *dest,
 	if (ret != CS_SUCCESS) return ret;
 	
 	if (is_krnl)
-	    memcpy_topc(dest, buf, npb);
+	    copy_to_pc(dest, buf, npb);
 	else
-	    memcpy_fromfs_topc(dest, buf, npb);
+	    copy_user_to_pc(dest, buf, npb);
 	
 	ret = page_write((u_short *)esr, (u_short *)dest, npb);
 	if (ret != CS_SUCCESS) return ret;
@@ -1123,7 +1130,7 @@ static int flash_request(dev_link_t *link, void *buf, mtd_request_t *req)
 ======================================================================*/
 
 static int flash_event(event_t event, int priority,
-		      event_callback_args_t *args)
+		       event_callback_args_t *args)
 {
     dev_link_t *link = args->client_data;
 
@@ -1173,7 +1180,7 @@ static int flash_event(event_t event, int priority,
 
 /*====================================================================*/
 
-int init_iflash2X_mtd(void)
+int init_module(void)
 {
     servinfo_t serv;
     
@@ -1194,7 +1201,6 @@ int init_iflash2X_mtd(void)
     return 0;
 }
 
-#ifdef MODULE
 void cleanup_module(void)
 {
 #ifdef PCMCIA_DEBUG
@@ -1205,4 +1211,3 @@ void cleanup_module(void)
     while (dev_list != NULL)
 	flash_detach(dev_list);
 }
-#endif /* MODULE */
