@@ -11,15 +11,11 @@
     
 ======================================================================*/
 
-#include "config.h"
-#include "k_compat.h"
+#include <pcmcia/config.h>
+#include <pcmcia/k_compat.h>
 
 /* #define PCMCIA_DEBUG 1 */
 
-#ifdef MODULE
-#define init_iflash2_mtd init_module
-#endif
-
 #include <linux/kernel.h>
 #include <linux/sched.h>
 #include <linux/ptrace.h>
@@ -34,20 +30,20 @@
 #include <asm/segment.h>
 #include <stdarg.h>
 
-#include "version.h"
-#include "cs_types.h"
-#include "cs.h"
-#include "bulkmem.h"
-#include "cisreg.h"
-#include "cistpl.h"
-#include "ds.h"
-#include "mem_op.h"
+#include <pcmcia/version.h>
+#include <pcmcia/cs_types.h>
+#include <pcmcia/cs.h>
+#include <pcmcia/bulkmem.h>
+#include <pcmcia/cisreg.h>
+#include <pcmcia/cistpl.h>
+#include <pcmcia/ds.h>
+#include <pcmcia/mem_op.h>
 #include "iflash.h"
 
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
 static char *version =
-"iflash2_mtd.c 1.20 1996/05/28 15:14:24 (David Hinds)";
+"iflash2_mtd.c 1.23 1996/12/21 19:30:51 (David Hinds)";
 #endif
 
 /*====================================================================*/
@@ -640,9 +636,9 @@ static int flash_read(dev_link_t *link, char *buf, mtd_request_t *req)
 	nb = (from+length > WINDOW_SIZE) ? WINDOW_SIZE-from : length;
 
 	if (req->Function & MTD_REQ_KERNEL)
-	    memcpy_frompc(buf, &dev->Base[from], nb);
+	    copy_from_pc(buf, &dev->Base[from], nb);
 	else
-	    memcpy_frompc_tofs(buf, &dev->Base[from], nb);
+	    copy_pc_to_user(buf, &dev->Base[from], nb);
 	
 	buf += nb;
 	from = 0;
@@ -697,12 +693,16 @@ static int basic_write(char *dest, char *buf,
     }
     else {
 	if (nb & 1) {
-	    ret = byte_write(dest, get_fs_byte(buf));
+	    char c;
+	    get_user(c, buf);
+	    ret = byte_write(dest, c);
 	    if (ret != CS_SUCCESS) return ret;
 	    dest++; buf++; nb--;
 	}
 	for (; nb != 0; dest += 2, buf += 2, nb -= 2) {
-	    ret = word_write((u_short *)dest, get_fs_word((u_short *)buf));
+	    u_short s;
+	    get_user(s, (u_short *)buf);
+	    ret = word_write((u_short *)dest, s);
 	    if (ret != CS_SUCCESS) return ret;
 	}
     }
@@ -1019,7 +1019,7 @@ static int flash_event(event_t event, int priority,
 
 /*====================================================================*/
 
-int init_iflash2_mtd(void)
+int init_module(void)
 {
     servinfo_t serv;
     
@@ -1040,7 +1040,6 @@ int init_iflash2_mtd(void)
     return 0;
 }
 
-#ifdef MODULE
 void cleanup_module(void)
 {
 #ifdef PCMCIA_DEBUG
@@ -1051,4 +1050,3 @@ void cleanup_module(void)
     while (dev_list != NULL)
 	flash_detach(dev_list);
 }
-#endif /* MODULE */
