@@ -123,12 +123,8 @@ Conditional Compilation Options
 Include Files
 ---------------------------------------------------------------------------- */
 
-#include "config.h"
-#include "k_compat.h"
-
-#ifdef MODULE
-#define init_nmclan_cs init_module
-#endif
+#include <pcmcia/config.h>
+#include <pcmcia/k_compat.h>
 
 #include <linux/kernel.h>
 #include <linux/sched.h>
@@ -149,12 +145,12 @@ Include Files
 #include <linux/if_arp.h>
 #include <linux/ioport.h>
 
-#include "version.h"
-#include "cs_types.h"
-#include "cs.h"
-#include "cisreg.h"
-#include "cistpl.h"
-#include "ds.h"
+#include <pcmcia/version.h>
+#include <pcmcia/cs_types.h>
+#include <pcmcia/cs.h>
+#include <pcmcia/cisreg.h>
+#include <pcmcia/cistpl.h>
+#include <pcmcia/ds.h>
 
 
 /* ----------------------------------------------------------------------------
@@ -1885,9 +1881,9 @@ static void set_multicast_list(struct device *dev, int num_addrs, void *addrs)
 
 
 /* ----------------------------------------------------------------------------
-init_nmclan_cs
+init_module
 ---------------------------------------------------------------------------- */
-int init_nmclan_cs(void)
+int init_module(void)
 {
   servinfo_t serv;
 #ifdef PCMCIA_DEBUG
@@ -1901,14 +1897,13 @@ int init_nmclan_cs(void)
   }
   register_pcmcia_driver(&dev_info, &nmclan_attach, &nmclan_detach);
   return 0;
-} /* init_nmclan_cs */
+} /* init_module */
 
 
 /* ----------------------------------------------------------------------------
 cleanup_module
 ---------------------------------------------------------------------------- */
 
-#ifdef MODULE
 void cleanup_module(void)
 {
 #ifdef PCMCIA_DEBUG
@@ -1919,4 +1914,3 @@ void cleanup_module(void)
   while (dev_list != NULL)
     nmclan_detach(dev_list);
 } /* cleanup_module */
-#endif /* MODULE */
