@@ -10,12 +10,8 @@
     
 ======================================================================*/
 
-#include "config.h"
-#include "k_compat.h"
-
-#ifdef MODULE
-#define init_pcmem_cs init_module
-#endif
+#include <pcmcia/config.h>
+#include <pcmcia/k_compat.h>
 
 #include <linux/kernel.h>
 #include <linux/sched.h>
@@ -31,12 +27,12 @@
 #include <asm/string.h>
 #include <stdarg.h>
 
-#include "version.h"
-#include "cs_types.h"
-#include "cs.h"
-#include "cistpl.h"
-#include "ds.h"
-#include "mem_op.h"
+#include <pcmcia/version.h>
+#include <pcmcia/cs_types.h>
+#include <pcmcia/cs.h>
+#include <pcmcia/cistpl.h>
+#include <pcmcia/ds.h>
+#include <pcmcia/mem_op.h>
 
 /* Major device #'s for pcmem device */
 static int major_dev = 0;
@@ -55,7 +51,7 @@ static int major_dev = 0;
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
 static char *version =
-"pcmem_cs.c 1.64 1996/09/15 04:13:12 (David Hinds)";
+"pcmem_cs.c 1.68 1996/12/21 19:30:54 (David Hinds)";
 #endif
 
 /*====================================================================*/
@@ -111,10 +107,10 @@ static int pcmem_lseek(struct inode *inode, struct file *file,
 		       off_t offset, int orig);
 static int pcmem_ioctl(struct inode *inode, struct file *file,
 		       u_int cmd, u_long arg);
-static int pcmem_read(struct inode *inode, struct file *file,
-		      char *buf, int count);
-static int pcmem_write(struct inode *inode, struct file *file,
-		       CONST char *buf, int count);
+static FS_SIZE_T pcmem_read(struct inode *inode, struct file *file,
+			    char *buf, U_FS_SIZE_T count);
+static FS_SIZE_T pcmem_write(struct inode *inode, struct file *file,
+			     CONST char *buf, U_FS_SIZE_T count);
 static int pcmem_open(struct inode *inode, struct file *file);
 static void pcmem_close(struct inode *inode, struct file *file);
 
@@ -586,8 +582,8 @@ static int pcmem_lseek(struct inode *inode, struct file *file,
     
 ======================================================================*/
 
-static int pcmem_read(struct inode *inode, struct file *file,
-		      char *buf, int count)
+static FS_SIZE_T pcmem_read(struct inode *inode, struct file *file,
+			    char *buf, U_FS_SIZE_T count)
 {
     int minor = MINOR(inode->i_rdev);
     dev_link_t *link;
@@ -644,11 +640,11 @@ static int pcmem_read(struct inode *inode, struct file *file,
 	nb = (from+count > WINDOW_SIZE) ? WINDOW_SIZE-from : count;
 	if (minor & 1) {
 	    for (i = 0; i < nb; i += 2, buf++)
-		put_fs_byte(readb(dev->Base+from+i), buf);
+		put_user(readb(dev->Base+from+i), buf);
 	    read += nb>>1;
 	}
 	else {
-	    memcpy_frompc_tofs(buf, dev->Base+from, nb);
+	    copy_pc_to_user(buf, dev->Base+from, nb);
 	    buf += nb;
 	    read += nb;
 	}
@@ -666,8 +662,8 @@ static int pcmem_read(struct inode *inode, struct file *file,
     
 ======================================================================*/
 
-static int pcmem_write(struct inode *inode, struct file *file,
-		       CONST char *buf, int count)
+static FS_SIZE_T pcmem_write(struct inode *inode, struct file *file,
+			     CONST char *buf, U_FS_SIZE_T count)
 {
     int minor = MINOR(inode->i_rdev);
     dev_link_t *link;
@@ -727,12 +723,15 @@ static int pcmem_write(struct inode *inode, struct file *file,
 	}
 	nb = (to+count > WINDOW_SIZE) ? WINDOW_SIZE-to : count;
 	if (minor & 1) {
-	    for (i = 0; i < nb; i += 2, buf++)
-		writeb(get_fs_byte(buf), dev->Base+to+i);
+	    for (i = 0; i < nb; i += 2, buf++) {
+		char c;
+		get_user(c, buf);
+		writeb(c, dev->Base+to+i);
+	    }
 	    wrote += nb>>1;
 	}
 	else {
-	    memcpy_fromfs_topc(dev->Base+to, buf, nb);
+	    copy_user_to_pc(dev->Base+to, buf, nb);
 	    buf += nb;
 	    wrote += nb;
 	}
@@ -770,7 +769,7 @@ static int pcmem_ioctl(struct inode *inode, struct file *file,
 	}
 	ret = verify_area(VERIFY_WRITE, (long *)arg, sizeof(long));
 	if (ret) break;
-	put_fs_long(dev->size/SECTOR_SIZE, (long *)arg);
+	put_user(dev->size/SECTOR_SIZE, (long *)arg);
 	break;
     default:
 	ret = -EINVAL;
@@ -830,7 +829,7 @@ static void do_pcmem_request(void)
 		ret = CardServices(MapMemPage, link->win, &mem);
 		if (ret != CS_SUCCESS) break;
 		nb = (from+len > WINDOW_SIZE) ? WINDOW_SIZE-from : len;
-		memcpy_frompc(buf, &dev->Base[from], nb);
+		copy_from_pc(buf, &dev->Base[from], nb);
 		mem.CardOffset += WINDOW_SIZE;
 	    }
 	    break;
@@ -840,7 +839,7 @@ static void do_pcmem_request(void)
 		ret = CardServices(MapMemPage, link->win, &mem);
 		if (ret != CS_SUCCESS) break;
 		nb = (from+len > WINDOW_SIZE) ? WINDOW_SIZE-from : len;
-		memcpy_topc(&dev->Base[from], buf, nb);
+		copy_to_pc(&dev->Base[from], buf, nb);
 		mem.CardOffset += WINDOW_SIZE;
 	    }
 	    break;
@@ -860,7 +859,7 @@ static void do_pcmem_request(void)
 
 /*====================================================================*/
 
-int init_pcmem_cs(void)
+int init_module(void)
 {
     servinfo_t serv;
     int i;
@@ -900,7 +899,6 @@ int init_pcmem_cs(void)
     return 0;
 }
 
-#ifdef MODULE
 void cleanup_module(void)
 {
     int i;
@@ -924,4 +922,3 @@ void cleanup_module(void)
 	}
     }
 }
-#endif /* MODULE */
