@@ -26,12 +26,8 @@
 
 ======================================================================*/
 
-#include "config.h"
-#include "k_compat.h"
-
-#ifdef MODULE
-#define init_pcnet_cs init_module
-#endif
+#include <pcmcia/config.h>
+#include <pcmcia/k_compat.h>
 
 #include <linux/kernel.h>
 #include <linux/sched.h>
@@ -46,13 +42,13 @@
 #include <linux/netdevice.h>
 #include <drivers/net/8390.h>
 
-#include "version.h"
-#include "cs_types.h"
-#include "cs.h"
-#include "cistpl.h"
-#include "ciscode.h"
-#include "ds.h"
-#include "cisreg.h"
+#include <pcmcia/version.h>
+#include <pcmcia/cs_types.h>
+#include <pcmcia/cs.h>
+#include <pcmcia/cistpl.h>
+#include <pcmcia/ciscode.h>
+#include <pcmcia/ds.h>
+#include <pcmcia/cisreg.h>
 
 #define PCNET_CMD	0x00
 #define PCNET_DATAPORT	0x10	/* NatSemi-defined port window offset. */
@@ -73,7 +69,7 @@ static char *if_names[] = { "Auto", "10baseT", "10base2"};
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
 static char *version =
-"pcnet_cs.c 1.42 1996/09/15 04:12:47 (David Hinds)";
+"pcnet_cs.c 1.44 1996/12/21 19:30:55 (David Hinds)";
 #endif
 
 /*====================================================================*/
@@ -146,43 +142,46 @@ typedef struct hw_info_t {
 #define HAS_IBM_MISC	0x10
 
 static hw_info_t hw_info[] = {
+    { "Accton EN2212", 0x0ff0, 0x00, 0x00, 0xe8, DELAY_OUTPUT },
+    { "Allied Telesis LA-PCM", 0x0ff0, 0x00, 0x00, 0xf4, 0 },
+    { "APEX MultiCard", 0x03f4, 0x00, 0x20, 0xe5, 0 },
+    { "Danpex EN-6200P2", 0x0110, 0x00, 0x40, 0xc7, 0 },
+    { "DataTrek NetCard", 0x0ff0, 0x00, 0x20, 0xe8, 0 },
     { "D-Link DE-650", 0x0040, 0x00, 0x80, 0xc8, 0 },
     { "EP-210 Ethernet", 0x0110, 0x00, 0x40, 0x33, 0 },
-    { "RPTI EP400", 0x0110, 0x00, 0x40, 0x95, 0 },
-    { "Danpex EN-6200P2", 0x0110, 0x00, 0x40, 0xc7, 0 },
-    { "PreMax PE-200", 0x07f0, 0x00, 0x20, 0xe0, 0 },
-    { "Accton EN2212", 0x0ff0, 0x00, 0x00, 0xe8, DELAY_OUTPUT },
-    { "Socket EA", 0x4000, 0x00, 0xc0, 0x1b,
-	  DELAY_OUTPUT | HAS_MISC_REG | USE_BIG_BUF },
-    { "Maxtech PCN2000", 0x5000, 0x00, 0x00, 0xe8, 0 },
-    { "Katron PE-520", 0x0110, 0x00, 0x40, 0xf6, 0 },
-    { "NE2000 Compatible", 0x0ff0, 0x00, 0xa0, 0x0c, 0 },
-    { "Panasonic VEL211", 0x0ff0, 0x00, 0x80, 0x45, 
+    { "Epson EEN10B", 0x0ff0, 0x00, 0x00, 0x48,
           USE_SHMEM | HAS_MISC_REG | HAS_IBM_MISC },
-    { "DataTrek NetCard", 0x0ff0, 0x00, 0x20, 0xe8, 0 },
+    { "ELECOM Laneed LD-CDWA", 0xb8, 0x08, 0x00, 0x42, 0 },
+    { "Grey Cell GCS2220", 0x0000, 0x00, 0x47, 0x43, 0 },
     { "IBM CCAE", 0x0ff0, 0x08, 0x00, 0x5a,
 	  USE_SHMEM | HAS_MISC_REG | HAS_IBM_MISC },
     { "IBM CCAE", 0x0ff0, 0x00, 0x04, 0xac,
           USE_SHMEM | HAS_MISC_REG | HAS_IBM_MISC },
+    { "IBM CCAE", 0x0ff0, 0x00, 0x06, 0x29,
+          USE_SHMEM | HAS_MISC_REG | HAS_IBM_MISC },
     { "IBM FME", 0x0374, 0x08, 0x00, 0x5a,
 	  USE_SHMEM | HAS_MISC_REG | HAS_IBM_MISC },
     { "IBM FME", 0x0374, 0x00, 0x04, 0xac,
           USE_SHMEM | HAS_MISC_REG | HAS_IBM_MISC },
     { "Hypertec Ethernet", 0x01c0, 0x00, 0x40, 0x4c, 0 },
+    { "Katron PE-520", 0x0110, 0x00, 0x40, 0xf6, 0 },
     { "Kingston KNE-PCM/x", 0x0ff0, 0x00, 0xc0, 0xf0,
 	  USE_SHMEM | HAS_MISC_REG | HAS_IBM_MISC },
     { "Kingston KNE-PCM/x", 0x0ff0, 0xe2, 0x0c, 0x0f,
 	  USE_SHMEM | HAS_MISC_REG | HAS_IBM_MISC },
+    { "Longshine LCS-8534", 0, 0x08, 0x00, 0x00, 0 },
+    { "Maxtech PCN2000", 0x5000, 0x00, 0x00, 0xe8, 0 },
+    { "NDC Instant-Link", 0x003a, 0x00, 0x80, 0xc6, 0 },
+    { "NE2000 Compatible", 0x0ff0, 0x00, 0xa0, 0x0c, 0 },
     { "Network General Sniffer", 0x0ff0, 0x00, 0x00, 0x65,
 	  USE_SHMEM | HAS_MISC_REG | HAS_IBM_MISC },
-    { "Grey Cell GCS2220", 0x0000, 0x00, 0x47, 0x43, 0 },
-    { "Allied Telesis LA-PCM", 0x0ff0, 0x00, 0x00, 0xf4, 0 },
-    { "SCM Ethernet", 0x0ff0, 0x00, 0x20, 0xcb, 0 },
-    { "NDC Instant-Link", 0x003a, 0x00, 0x80, 0xc6, 0 },
-    { "Epson EEN10B", 0x0ff0, 0x00, 0x00, 0x48,
+    { "Panasonic VEL211", 0x0ff0, 0x00, 0x80, 0x45, 
           USE_SHMEM | HAS_MISC_REG | HAS_IBM_MISC },
-    { "Longshine LCS-8534", 0, 0x08, 0x00, 0x00, 0 },
-    { "ELECOM Laneed LD-CDWA", 0xb8, 0x08, 0x00, 0x42, 0 }
+    { "PreMax PE-200", 0x07f0, 0x00, 0x20, 0xe0, 0 },
+    { "RPTI EP400", 0x0110, 0x00, 0x40, 0x95, 0 },
+    { "SCM Ethernet", 0x0ff0, 0x00, 0x20, 0xcb, 0 },
+    { "Socket EA", 0x4000, 0x00, 0xc0, 0x1b,
+	  DELAY_OUTPUT | HAS_MISC_REG | USE_BIG_BUF }
 };
 
 #define NR_INFO		(sizeof(hw_info)/sizeof(hw_info_t))
@@ -1282,7 +1281,7 @@ config_failed:
 
 /*====================================================================*/
 
-int init_pcnet_cs(void)
+int init_module(void)
 {
     servinfo_t serv;
 #ifdef PCMCIA_DEBUG
@@ -1299,7 +1298,6 @@ int init_pcnet_cs(void)
     return 0;
 }
 
-#ifdef MODULE
 void cleanup_module(void)
 {
 #ifdef PCMCIA_DEBUG
@@ -1310,5 +1308,3 @@ void cleanup_module(void)
     while (dev_list != NULL)
 	pcnet_detach(dev_list);
 }
-#endif /* MODULE */
-
