@@ -6,12 +6,8 @@
     
 ======================================================================*/
 
-#include "config.h"
-#include "k_compat.h"
-
-#ifdef MODULE
-#define init_serial_cs init_module
-#endif
+#include <pcmcia/config.h>
+#include <pcmcia/k_compat.h>
 
 #include <linux/kernel.h>
 #include <linux/sched.h>
@@ -25,18 +21,18 @@
 #include <asm/io.h>
 #include <asm/system.h>
 
-#include "version.h"
-#include "cs_types.h"
-#include "cs.h"
-#include "cistpl.h"
-#include "ciscode.h"
-#include "ds.h"
-#include "cisreg.h"
+#include <pcmcia/version.h>
+#include <pcmcia/cs_types.h>
+#include <pcmcia/cs.h>
+#include <pcmcia/cistpl.h>
+#include <pcmcia/ciscode.h>
+#include <pcmcia/ds.h>
+#include <pcmcia/cisreg.h>
 
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
 static char *version =
-"serial_cs.c 1.73 1996/10/03 02:06:47 (David Hinds)";
+"serial_cs.c 1.78 1996/12/22 03:49:53 (David Hinds)";
 #endif
 
 /*====================================================================*/
@@ -234,8 +230,8 @@ static int setup_serial(serial_info_t *info, ioaddr_t port, int irq)
     
     info->line[info->ndev] = line;
     sprintf(info->node[info->ndev].dev_name, "ttyS%d", line);
-    info->node[0].major = TTY_MAJOR;
-    info->node[0].minor = 0x40+line;
+    info->node[info->ndev].major = TTY_MAJOR;
+    info->node[info->ndev].minor = 0x40+line;
     if (info->ndev > 0)
 	info->node[info->ndev-1].next = &info->node[info->ndev];
     info->ndev++;
@@ -245,60 +241,68 @@ static int setup_serial(serial_info_t *info, ioaddr_t port, int irq)
 
 /*====================================================================*/
 
+static int get_tuple(int fn, client_handle_t handle, tuple_t *tuple,
+		     cisparse_t *parse)
+{
+    int i;
+    i = CardServices(fn, handle, tuple);
+    if (i != CS_SUCCESS) return i;
+    i = CardServices(GetTupleData, handle, tuple);
+    if (i != CS_SUCCESS) return i;
+    return CardServices(ParseTuple, handle, tuple, parse);
+}
+
+#define first_tuple(a, b, c) get_tuple(GetFirstTuple, a, b, c)
+#define next_tuple(a, b, c) get_tuple(GetNextTuple, a, b, c)
+
+/*====================================================================*/
+
 static int simple_config(dev_link_t *link)
 {
     static ioaddr_t base[5] = { 0x3f8, 0x2f8, 0x3e8, 0x2e8, 0x0 };
     client_handle_t handle = link->handle;
     serial_info_t *info = link->priv;
     tuple_t tuple;
-    u_short buf[128];
+    u_char buf[256];
     cisparse_t parse;
     cistpl_cftable_entry_t *cf = &parse.cftable_entry;
     int i, j;
-    
+
     /* First pass: look for a config entry that looks normal. */
     tuple.TupleData = (cisdata_t *)buf;
     tuple.TupleOffset = 0; tuple.TupleDataMax = 255;
     tuple.Attributes = 0;
     tuple.DesiredTuple = CISTPL_CFTABLE_ENTRY;
-    i = CardServices(GetFirstTuple, handle, &tuple);
+    i = first_tuple(handle, &tuple, &parse);
     while (i == CS_SUCCESS) {
-	i = CardServices(GetTupleData, handle, &tuple);
-	if (i != CS_SUCCESS) break;
-	i = CardServices(ParseTuple, handle, &tuple, &parse);
-	if ((i == CS_SUCCESS) && (cf->io.nwin > 0) &&
-	    ((cf->io.win[0].base & 0xf) == 8)) {
+	if ((cf->io.nwin > 0) && ((cf->io.win[0].base & 0xf) == 8)) {
 	    link->conf.ConfigIndex = cf->index;
 	    link->io.BasePort1 = cf->io.win[0].base;
 	    i = CardServices(RequestIO, link->handle, &link->io);
-	    if (i == CS_SUCCESS) break;
+	    if (i == CS_SUCCESS) goto found_port;
 	}
-	i = CardServices(GetNextTuple, handle, &tuple);
+	i = next_tuple(handle, &tuple, &parse);
     }
     
     /* Second pass: try to find an entry that isn't picky about
        its base address, then try to grab any standard serial port
        address, and finally try to get any free port. */
-    if (i != CS_SUCCESS) {
-	i = CardServices(GetFirstTuple, handle, &tuple);
-	while (i == CS_SUCCESS) {
-	    i = CardServices(GetTupleData, handle, &tuple);
-	    if (i != CS_SUCCESS) break;
-	    i = CardServices(ParseTuple, handle, &tuple, &parse);
-	    if ((i == CS_SUCCESS) && (cf->io.nwin > 0) &&
-		((cf->io.flags & CISTPL_IO_LINES_MASK) <= 3)) {
-		link->conf.ConfigIndex = cf->index;
-		for (j = 0; j < 5; j++) {
-		    link->io.BasePort1 = base[j];
-		    i = CardServices(RequestIO, link->handle,
-				     &link->io);
-		    if (i == CS_SUCCESS) break;
-		}
-		if (i == CS_SUCCESS) break;
+    i = first_tuple(handle, &tuple, &parse);
+    while (i == CS_SUCCESS) {
+	if ((cf->io.nwin > 0) &&
+	    ((cf->io.flags & CISTPL_IO_LINES_MASK) <= 3)) {
+	    link->conf.ConfigIndex = cf->index;
+	    for (j = 0; j < 5; j++) {
+		link->io.BasePort1 = base[j];
+		i = CardServices(RequestIO, link->handle,
+				 &link->io);
+		if (i == CS_SUCCESS) goto found_port;
 	    }
-	    i = CardServices(GetNextTuple, handle, &tuple);
 	}
+	i = next_tuple(handle, &tuple, &parse);
     }
+
+found_port:
     if (i != CS_SUCCESS) {
 	cs_error(RequestIO, i);
 	return -1;
@@ -328,6 +332,7 @@ static int slave_config(dev_link_t *link)
     ioaddr_t port;
     int i;
 
+    printk("slave_config()\n");
     /* If the card is already configured, look up the port and irq */
     i = CardServices(GetConfigurationInfo, handle, &config);
     if ((i == CS_SUCCESS) &&
@@ -348,7 +353,7 @@ static int dual_config(dev_link_t *link)
     client_handle_t handle = link->handle;
     serial_info_t *info = link->priv;
     tuple_t tuple;
-    u_short buf[128];
+    u_char buf[256];
     cisparse_t parse;
     cistpl_cftable_entry_t *cf = &parse.cftable_entry;
     int i;
@@ -358,17 +363,15 @@ static int dual_config(dev_link_t *link)
     tuple.TupleOffset = 0; tuple.TupleDataMax = 255;
     tuple.Attributes = 0;
     tuple.DesiredTuple = CISTPL_CFTABLE_ENTRY;
-    i = CardServices(GetFirstTuple, handle, &tuple);
+    i = first_tuple(handle, &tuple, &parse);
     while (i == CS_SUCCESS) {
-	i = CardServices(GetTupleData, handle, &tuple);
-	if (i != CS_SUCCESS) break;
-	i = CardServices(ParseTuple, handle, &tuple, &parse);
-	if ((i == CS_SUCCESS) && (cf->io.nwin == 1)) {
+	if ((cf->io.nwin == 1) && (cf->io.win[0].len == 16)) {
 	    link->conf.ConfigIndex = cf->index;
+	    link->io.BasePort1 = cf->io.win[0].base;
 	    i = CardServices(RequestIO, link->handle, &link->io);
 	    if (i == CS_SUCCESS) break;
 	}
-	i = CardServices(GetNextTuple, handle, &tuple);
+	i = next_tuple(handle, &tuple, &parse);
     }
     if (i != CS_SUCCESS) {
 	cs_error(RequestIO, i);
@@ -409,6 +412,7 @@ void serial_config(dev_link_t *link)
     tuple_t tuple;
     u_short buf[128];
     cisparse_t parse;
+    cistpl_cftable_entry_t *cf = &parse.cftable_entry;
     int i;
 
     sti();
@@ -425,12 +429,8 @@ void serial_config(dev_link_t *link)
     tuple.Attributes = 0;
     /* Get configuration register information */
     tuple.DesiredTuple = CISTPL_CONFIG;
-    i = CardServices(GetFirstTuple, handle, &tuple);
+    i = first_tuple(handle, &tuple, &parse);
     CS_EXIT_TEST(i, GetFirstTuple, config_failed);
-    i = CardServices(GetTupleData, handle, &tuple);
-    CS_EXIT_TEST(i, GetTupleData, config_failed);
-    i = CardServices(ParseTuple, handle, &tuple, &parse);
-    CS_EXIT_TEST(i, ParseTuple, config_failed);
     link->conf.ConfigBase = parse.config.base;
     link->conf.Present = parse.config.rmask[0];
     
@@ -440,8 +440,7 @@ void serial_config(dev_link_t *link)
     /* Is this a multifunction card? */
     tuple.DesiredTuple = CISTPL_MANFID;
     tuple.Attributes = TUPLE_RETURN_COMMON;
-    if ((CardServices(GetFirstTuple, handle, &tuple) == CS_SUCCESS) &&
-	(CardServices(GetTupleData, handle, &tuple) == CS_SUCCESS)) {
+    if (first_tuple(handle, &tuple, &parse) == CS_SUCCESS) {
 	info->manfid = buf[0];
 	for (i = 0; i < MULTI_COUNT; i++)
 	    if ((buf[0] == multi_id[i].manfid) &&
@@ -453,6 +452,15 @@ void serial_config(dev_link_t *link)
 	}
     }
 
+    /* Another check for dual-serial cards */
+    tuple.DesiredTuple = CISTPL_CFTABLE_ENTRY;
+    if ((info->multi == 0) &&
+	(first_tuple(handle, &tuple, &parse) == CS_SUCCESS) &&
+	(((cf->io.nwin == 1) && (cf->io.win[0].len == 16)) ||
+	 ((cf->io.nwin == 2) && (cf->io.win[0].len == 8) &&
+	  (cf->io.win[1].len == 8))))
+	info->multi = 2;
+
     if (info->multi == 2)
 	dual_config(link);
     else if (info->slave)
@@ -576,7 +584,7 @@ static int serial_event(event_t event, int priority,
 
 /*====================================================================*/
 
-int init_serial_cs(void)
+int init_module(void)
 {
     servinfo_t serv;
 #ifdef PCMCIA_DEBUG
@@ -593,7 +601,6 @@ int init_serial_cs(void)
     return 0;
 }
 
-#ifdef MODULE
 void cleanup_module(void)
 {
 #ifdef PCMCIA_DEBUG
@@ -604,4 +611,3 @@ void cleanup_module(void)
     while (dev_list != NULL)
 	serial_detach(dev_list);
 }
-#endif /* MODULE */
