@@ -18,12 +18,8 @@
 
 ======================================================================*/
 
-#include "config.h"
-#include "k_compat.h"
-
-#ifdef MODULE
-#define init_smc91c92_cs init_module
-#endif
+#include <pcmcia/config.h>
+#include <pcmcia/k_compat.h>
 
 #include <ctype.h>
 #include <linux/kernel.h>
@@ -43,13 +39,13 @@
 #include <linux/if_arp.h>
 #include <linux/ioport.h>
 
-#include "version.h"
-#include "cs_types.h"
-#include "cs.h"
-#include "cistpl.h"
-#include "cisreg.h"
-#include "ciscode.h"
-#include "ds.h"
+#include <pcmcia/version.h>
+#include <pcmcia/cs_types.h>
+#include <pcmcia/cs.h>
+#include <pcmcia/cistpl.h>
+#include <pcmcia/cisreg.h>
+#include <pcmcia/ciscode.h>
+#include <pcmcia/ds.h>
 
 /*====================================================================*/
 
@@ -103,6 +99,7 @@ struct smc_private {
     dev_node_t			node;
     struct sk_buff		*saved_skb;
     int				packets_waiting;
+    caddr_t			base;
 };
 
 /* Special definitions for Megahertz multifunction cards */
@@ -409,6 +406,125 @@ static int cvt_ascii_address(struct device *dev, char *s)
 
 /*====================================================================*/
 
+static int get_tuple(int fn, client_handle_t handle, tuple_t *tuple,
+		     cisparse_t *parse)
+{
+    int i;
+    i = CardServices(fn, handle, tuple);
+    if (i != CS_SUCCESS) return i;
+    i = CardServices(GetTupleData, handle, tuple);
+    if (i != CS_SUCCESS) return i;
+    return CardServices(ParseTuple, handle, tuple, parse);
+}
+
+#define first_tuple(a, b, c) get_tuple(GetFirstTuple, a, b, c)
+#define next_tuple(a, b, c) get_tuple(GetNextTuple, a, b, c)
+
+/*======================================================================
+
+    Configuration stuff for Megahertz cards
+
+    mhz_3288_power() is used to power up a 3288's ethernet chip.
+    mhz_mfc_config() handles socket setup for multifunction (1144
+    and 3288) cards.  mhz_setup() gets a card's hardware ethernet
+    address.
+    
+======================================================================*/
+
+static void busy_loop(u_long len)
+{
+    u_long timeout = jiffies + len;
+    u_long flags;
+    save_flags(flags);
+    sti();
+    while (timeout >= jiffies) ;
+    restore_flags(flags);
+}
+
+static int mhz_3288_power(dev_link_t *link)
+{
+    struct device *dev = link->priv;
+    struct smc_private *lp = dev->priv;
+    u_char tmp;
+    
+    /* Read the ISR twice... */
+    readb(lp->base+MEGAHERTZ_ISR);
+    udelay(5);
+    readb(lp->base+MEGAHERTZ_ISR);
+
+    /* Pause 200ms... */
+    busy_loop(HZ/5);
+    
+    /* Now read and write the COR... */
+    tmp = readb(lp->base+link->conf.ConfigBase+CISREG_COR);
+    udelay(5);
+    writeb(tmp, lp->base+link->conf.ConfigBase+CISREG_COR);
+
+    return 0;
+}
+
+static int mhz_mfc_config(dev_link_t *link)
+{
+    struct device *dev = link->priv;
+    struct smc_private *lp = dev->priv;
+    tuple_t tuple;
+    cisparse_t parse;
+    u_char buf[255];
+    cistpl_cftable_entry_t *cf = &parse.cftable_entry;
+    win_req_t req;
+    memreq_t mem;
+    int i, k;
+
+    link->conf.Attributes |= CONF_ENABLE_SPKR;
+    link->conf.Status = CCSR_AUDIO_ENA;
+    link->irq.Attributes =
+	IRQ_TYPE_DYNAMIC_SHARING|IRQ_FIRST_SHARED|IRQ_HANDLE_PRESENT;
+    link->io.Attributes2 = IO_DATA_PATH_WIDTH_8;
+    link->io.NumPorts2 = 8;
+
+    tuple.Attributes = 0;
+    tuple.TupleData = (cisdata_t *)buf;
+    tuple.TupleDataMax = sizeof(buf);
+    tuple.TupleOffset = 0;
+    tuple.DesiredTuple = CISTPL_CFTABLE_ENTRY;
+
+    i = first_tuple(link->handle, &tuple, &parse);
+    /* The Megahertz combo cards have modem-like CIS entries, so
+       we have to explicitly try a bunch of port combinations. */
+    while (i == CS_SUCCESS) {
+	link->conf.ConfigIndex = cf->index;
+	link->io.BasePort2 = cf->io.win[0].base;
+	for (k = 0x300; k < 0x380; k += 0x20) {
+	    link->io.BasePort1 = k;
+	    i = CardServices(RequestIO, link->handle, &link->io);
+	    if (i == CS_SUCCESS) break;
+	}
+	if (i == CS_SUCCESS) break;
+	i = next_tuple(link->handle, &tuple, &parse);
+    }
+    if (i != CS_SUCCESS)
+	return i;
+    dev->base_addr = link->io.BasePort1;
+    
+    /* Allocate a memory window, for accessing the ISR */
+    req.Attributes = WIN_DATA_WIDTH_8|WIN_MEMORY_TYPE_AM|WIN_ENABLE;
+    req.Base = NULL;
+    req.Size = 0x1000;
+    req.AccessSpeed = 0;
+    link->win = (window_handle_t)link->handle;
+    i = CardServices(RequestWindow, &link->win, &req);
+    if (i != CS_SUCCESS)
+	return i;
+    lp->base = req.Base;
+    mem.CardOffset = mem.Page = 0;
+    i = CardServices(MapMemPage, link->win, &mem);
+    
+    if ((i == CS_SUCCESS) && (lp->cardid == PRODID_MEGAHERTZ_EM3288))
+	mhz_3288_power(link);
+    
+    return i;
+}
+
 static int mhz_setup(dev_link_t *link)
 {
     client_handle_t handle = link->handle;
@@ -416,26 +532,25 @@ static int mhz_setup(dev_link_t *link)
     tuple_t tuple;
     cisparse_t parse;
     u_char buf[255], *station_addr;
-    
-    /* Read the station address from the CIS.  It is stored as the last
-       (fourth) string in the Version 1 Version/ID tuple. */
+
     tuple.Attributes = 0;
-    tuple.DesiredTuple = CISTPL_VERS_1;
     tuple.TupleData = buf;
     tuple.TupleDataMax = sizeof(buf);
     tuple.TupleOffset = 0;
-    if (CardServices(GetFirstTuple, handle, &tuple) != CS_SUCCESS)
+
+    /* Read the station address from the CIS.  It is stored as the last
+       (fourth) string in the Version 1 Version/ID tuple. */
+    tuple.DesiredTuple = CISTPL_VERS_1;
+    if (first_tuple(handle, &tuple, &parse) != CS_SUCCESS)
 	return -1;
     /* Ugh -- the EM1144 card has two VERS_1 tuples!?! */
-    if (CardServices(GetNextTuple, handle, &tuple) != CS_SUCCESS)
-	CardServices(GetFirstTuple, handle, &tuple);
-    if (CardServices(GetTupleData, handle, &tuple) != CS_SUCCESS)
-	return -1;
-    if (CardServices(ParseTuple, handle, &tuple, &parse) != CS_SUCCESS)
-	return -1;
-    station_addr = parse.version_1.str + parse.version_1.ofs[3];
-    if (cvt_ascii_address(dev, station_addr) == 0)
-	return 0;
+    if (next_tuple(handle, &tuple, &parse) != CS_SUCCESS)
+	first_tuple(handle, &tuple, &parse);
+    if (parse.version_1.ns > 3) {
+	station_addr = parse.version_1.str + parse.version_1.ofs[3];
+	if (cvt_ascii_address(dev, station_addr) == 0)
+	    return 0;
+    }
 
     /* Another possibility: for the EM3288, in a special tuple */
     tuple.DesiredTuple = 0x81;
@@ -450,52 +565,85 @@ static int mhz_setup(dev_link_t *link)
     return -1;
 }
 
-static void busy_loop(u_long len)
-{
-    u_long timeout = jiffies + len;
-    u_long flags;
-    save_flags(flags);
-    sti();
-    while (timeout >= jiffies) ;
-    restore_flags(flags);
-}
+/*====================================================================*/
 
-static int mhz_3288_power(dev_link_t *link)
+static int smc_setup(dev_link_t *link)
 {
-    win_req_t req;
-    memreq_t mem;
-    volatile u_char *attr;
-    u_char tmp;
+    client_handle_t handle = link->handle;
+    struct device *dev = link->priv;
+    tuple_t tuple;
+    cisparse_t parse;
+    cistpl_lan_node_id_t *node_id;
+    u_char buf[255], *station_addr;
+    int i;
     
-    req.Attributes = WIN_DATA_WIDTH_8|WIN_MEMORY_TYPE_AM|WIN_ENABLE;
-    req.Base = NULL;
-    req.Size = 0x1000;
-    req.AccessSpeed = 0;
-    link->win = (window_handle_t)link->handle;
-    if (CardServices(RequestWindow, &link->win, &req) != CS_SUCCESS)
+    tuple.Attributes = 0;
+    tuple.TupleData = buf;
+    tuple.TupleDataMax = sizeof(buf);
+    tuple.TupleOffset = 0;
+    
+    /* Read the station address from the CIS.  It is stored in the
+       third string in the Version 1 Version/ID tuple. */
+    tuple.DesiredTuple = CISTPL_VERS_1;
+    if (first_tuple(handle, &tuple, &parse) != CS_SUCCESS)
 	return -1;
-    mem.Page = mem.CardOffset = 0;
-    if (CardServices(MapMemPage, link->win, &mem) != CS_SUCCESS)
+    station_addr = parse.version_1.str + parse.version_1.ofs[2];
+    if (cvt_ascii_address(dev, station_addr) == 0)
+	return 0;
+
+    /* Another possibility: in the LAN function extension tuples */
+    tuple.DesiredTuple = CISTPL_FUNCE;
+    i = first_tuple(handle, &tuple, &parse);
+    while (i == CS_SUCCESS) {
+	if (parse.funce.type == CISTPL_FUNCE_LAN_NODE_ID)
+	    break;
+	i = next_tuple(handle, &tuple, &parse);
+    }
+    if (i != CS_SUCCESS)
 	return -1;
-
-    /* Read the ISR twice... */
-    attr = req.Base;
-    tmp = attr[MEGAHERTZ_ISR];
-    udelay(5);
-    tmp = attr[MEGAHERTZ_ISR];
-    /* Pause 200ms... */
-    busy_loop(HZ/5);
-    /* Now read and write the COR... */
-    tmp = attr[link->conf.ConfigBase + CISREG_COR];
-    udelay(5);
-    attr[link->conf.ConfigBase + CISREG_COR] = tmp;
-
-    CardServices(ReleaseWindow, link->win);
-    return 0;
+    node_id = (cistpl_lan_node_id_t *)parse.funce.data;
+    if (node_id->nb == 6) {
+	for (i = 0; i < 6; i++)
+	    dev->dev_addr[i] = node_id->id[i];
+	return 0;
+    }
+    return -1;
 }
 
 /*====================================================================*/
 
+static int osi_config(dev_link_t *link)
+{
+    struct device *dev = link->priv;
+    static u_short com[4] = { 0x3f8, 0x2f8, 0x3e8, 0x2e8 };
+    int i, j;
+    
+    link->conf.Attributes |= CONF_ENABLE_SPKR;
+    link->conf.Status = CCSR_AUDIO_ENA;
+    link->irq.Attributes =
+	IRQ_TYPE_DYNAMIC_SHARING|IRQ_FIRST_SHARED|IRQ_HANDLE_PRESENT;
+    link->io.NumPorts1 = 64;
+    link->io.Attributes2 = IO_DATA_PATH_WIDTH_8;
+    link->io.NumPorts2 = 8;
+    
+    /* Enable Hard Decode, LAN, Modem */
+    link->conf.ConfigIndex = 0x23;
+    
+    for (j = 0; j < 4; j++) {
+	link->io.BasePort2 = com[j];
+	i = CardServices(RequestIO, link->handle, &link->io);
+	if (i == CS_SUCCESS) break;
+    }
+    if (i != CS_SUCCESS) {
+	/* Fallback: turn off hard decode */
+	link->conf.ConfigIndex = 0x03;
+	link->io.NumPorts2 = 0;
+	i = CardServices(RequestIO, link->handle, &link->io);
+    }
+    dev->base_addr = link->io.BasePort1 + 0x10;
+    return i;
+}
+
 static int osi_setup(dev_link_t *link)
 {
     client_handle_t handle = link->handle;
@@ -504,12 +652,13 @@ static int osi_setup(dev_link_t *link)
     u_char buf[255];
     int i;
     
-    /* Read the station address from tuple 0x90, subtuple 0x04 */
     tuple.Attributes = 0;
-    tuple.DesiredTuple = 0x90;
     tuple.TupleData = buf;
     tuple.TupleDataMax = sizeof(buf);
     tuple.TupleOffset = 0;
+    
+    /* Read the station address from tuple 0x90, subtuple 0x04 */
+    tuple.DesiredTuple = 0x90;
     i = CardServices(GetFirstTuple, handle, &tuple);
     while (i == CS_SUCCESS) {
 	i = CardServices(GetTupleData, handle, &tuple);
@@ -548,9 +697,8 @@ static void smc91c92_config(dev_link_t *link)
     struct smc_private *lp = dev->priv;
     tuple_t tuple;
     cisparse_t parse;
-    u_short ioaddr, buf[32];
-    int i, j, k;
-    static u_short com[4] = { 0x03f8, 0x2f8, 0x3e8, 0x2e8 };
+    u_short buf[32];
+    int i, j;
 
 #ifdef PCMCIA_DEBUG
     if (pc_debug > 2)
@@ -563,79 +711,26 @@ static void smc91c92_config(dev_link_t *link)
     tuple.TupleOffset = 0;
 
     tuple.DesiredTuple = CISTPL_CONFIG;
-    i = CardServices(GetFirstTuple, handle, &tuple);
-    CS_EXIT_TEST(i, GetFirstTuple, config_failed);
-    i = CardServices(GetTupleData, handle, &tuple);
-    CS_EXIT_TEST(i, GetTupleData, config_failed);
-    i = CardServices(ParseTuple, handle, &tuple, &parse);
+    i = first_tuple(handle, &tuple, &parse);
     CS_EXIT_TEST(i, ParseTuple, config_failed);
     link->conf.ConfigBase = parse.config.base;
     link->conf.Present = parse.config.rmask[0];
     
     tuple.DesiredTuple = CISTPL_MANFID;
-    i = CardServices(GetFirstTuple, handle, &tuple);
+    i = first_tuple(handle, &tuple, &parse);
     CS_EXIT_TEST(i, GetFirstTuple, config_failed);
-    i = CardServices(GetTupleData, handle, &tuple);
-    CS_EXIT_TEST(i, GetTupleData, config_failed);
     lp->manfid = buf[0];
     lp->cardid = buf[1];
     
     /* Configure card */
     link->state |= DEV_CONFIG;
 
-    if (lp->manfid == MANFID_OSITECH) {
-	link->conf.Attributes |= CONF_ENABLE_SPKR;
-	link->conf.Status = CCSR_AUDIO_ENA;
-	link->irq.Attributes =
-	    IRQ_TYPE_DYNAMIC_SHARING|IRQ_FIRST_SHARED|IRQ_HANDLE_PRESENT;
-	link->io.NumPorts1 = 64;
-	link->io.Attributes2 = IO_DATA_PATH_WIDTH_8;
-	link->io.NumPorts2 = 8;
-	/* Enable Hard Decode, LAN, Modem */
-	link->conf.ConfigIndex = 0x23;
-	for (j = 0; j < 4; j++) {
-	    link->io.BasePort2 = com[j];
-	    i = CardServices(RequestIO, link->handle, &link->io);
-	    if (i == CS_SUCCESS) break;
-	}
-	if (i != CS_SUCCESS) {
-	    /* Fallback: turn off hard decode */
-	    link->conf.ConfigIndex = 0x03;
-	    link->io.NumPorts2 = 0;
-	    i = CardServices(RequestIO, link->handle, &link->io);
-	}
-    }
+    if (lp->manfid == MANFID_OSITECH)
+	i = osi_config(link);
     else if ((lp->manfid == MANFID_MEGAHERTZ) &&
 	     ((lp->cardid == PRODID_MEGAHERTZ_VARIOUS) ||
-	      (lp->cardid == PRODID_MEGAHERTZ_EM3288))) {
-	cistpl_cftable_entry_t *cf = &parse.cftable_entry;
-	
-	link->conf.Attributes |= CONF_ENABLE_SPKR;
-	link->conf.Status = CCSR_AUDIO_ENA;
-	link->irq.Attributes =
-	    IRQ_TYPE_DYNAMIC_SHARING|IRQ_FIRST_SHARED|IRQ_HANDLE_PRESENT;
-	link->io.Attributes2 = IO_DATA_PATH_WIDTH_8;
-	link->io.NumPorts2 = 8;
-	tuple.DesiredTuple = CISTPL_CFTABLE_ENTRY;
-	i = CardServices(GetFirstTuple, handle, &tuple);
-	/* The Megahertz combo cards have modem-like CIS entries, so
-	   we have to explicitly try a bunch of port combinations. */
-	for (j = 0; j < 4; j++) {
-	    i = CardServices(GetTupleData, handle, &tuple);
-	    if (i != CS_SUCCESS) break;
-	    i = CardServices(ParseTuple, handle, &tuple, &parse);
-	    if (i != CS_SUCCESS) break;
-	    link->conf.ConfigIndex = cf->index;
-	    link->io.BasePort2 = cf->io.win[0].base;
-	    for (k = 0x300; k < 0x380; k += 0x20) {
-		link->io.BasePort1 = k;
-		i = CardServices(RequestIO, link->handle, &link->io);
-		if (i == CS_SUCCESS) break;
-	    }
-	    if (i == CS_SUCCESS) break;
-	    i = CardServices(GetNextTuple, handle, &tuple);
-	}
-    }
+	      (lp->cardid == PRODID_MEGAHERTZ_EM3288)))
+	i = mhz_mfc_config(link);
     else {
 	link->io.NumPorts1 = 16;
 	link->conf.ConfigIndex = 0x01;
@@ -644,23 +739,16 @@ static void smc91c92_config(dev_link_t *link)
 	    i = CardServices(RequestIO, link->handle, &link->io);
 	    if (i == CS_SUCCESS) break;
 	}
+	dev->base_addr = link->io.BasePort1;
     }
     CS_EXIT_TEST(i, RequestIO, config_failed);
     
-    if ((lp->manfid == MANFID_MEGAHERTZ) &&
-	(lp->cardid == PRODID_MEGAHERTZ_EM3288))
-	mhz_3288_power(link);
-    
     i = CardServices(RequestIRQ, link->handle, &link->irq);
     CS_EXIT_TEST(i, RequestIRQ, config_failed);
     i = CardServices(RequestConfiguration, link->handle, &link->conf);
     CS_EXIT_TEST(i, RequestConfiguration, config_failed);
     
     dev->irq = link->irq.AssignedIRQ;
-    if (lp->manfid == MANFID_OSITECH)
-	ioaddr = dev->base_addr = link->io.BasePort1 + 0x10;
-    else
-	ioaddr = dev->base_addr = link->io.BasePort1;
 
     if ((if_port == 1) || (if_port == 2))
 	dev->if_port = if_port;
@@ -676,6 +764,8 @@ static void smc91c92_config(dev_link_t *link)
     switch (lp->manfid) {
     case MANFID_OSITECH:
 	i = osi_setup(link); break;
+    case MANFID_SMC:
+	i = smc_setup(link); break;
     case 0x128: /* For broken Megahertz cards */
     case MANFID_MEGAHERTZ:
 	i = mhz_setup(link); break;
@@ -743,6 +833,8 @@ static void smc91c92_release(u_long arg)
     CardServices(ReleaseConfiguration, link->handle);
     CardServices(ReleaseIO, link->handle, &link->io);
     CardServices(ReleaseIRQ, link->handle, &link->irq);
+    if (link->win)
+	CardServices(ReleaseWindow, link->win);
     if (dev->irq != 0)
 	irq2dev_map[dev->irq] = NULL;
     
@@ -769,7 +861,7 @@ static int smc91c92_event(event_t event, int priority,
     
 #ifdef PCMCIA_DEBUG
     if (pc_debug)
-	printk(KERN_DEBUG "smc91c92_event()\n");
+	printk(KERN_DEBUG "smc91c92_event(0x%04x)\n", event);
 #endif
 
     switch (event) {
@@ -1177,7 +1269,8 @@ static void smc_eph_irq(struct device *dev)
 	       " 10baseT.\n", dev->name);
     } else if ((dev->if_port == 1) && (ephs & 0x4000) == 0) {
 	dev->if_port = 2;
-	printk(KERN_INFO "%s: 10baseT link beat lost.\n", dev->name);
+	printk(KERN_INFO "%s: Link beat lost -- switching to"
+	       " 10base2.\n", dev->name);
     } else goto skip;
     outw(CFG_NO_WAIT | CFG_16BIT | CFG_STATIC |
 	 (lp->manfid == MANFID_OSITECH ?
@@ -1233,12 +1326,7 @@ static void smc_interrupt IRQ(int irq, void *instance, struct pt_regs *regs)
 		   "/ejected device.\n", dev->name, irq);
 	dev->interrupt = 0;
 #endif
-	if (lp->manfid == MANFID_OSITECH) {
-	    /* Retrigger interrupt if needed */
-	    outb(0x00, ioaddr - 0x10 + OSITECH_ISR);
-	    outb(0x03, ioaddr - 0x10 + OSITECH_ISR);
-	}
-	return;
+	goto irq_done;
     }
     
     SMC_SELECT_BANK(2);
@@ -1312,11 +1400,19 @@ static void smc_interrupt IRQ(int irq, void *instance, struct pt_regs *regs)
 	       dev->name, irq);
 #endif
 
+irq_done:
+    
     if (lp->manfid == MANFID_OSITECH) {
 	/* Retrigger interrupt if needed */
 	outb(0x00, ioaddr - 0x10 + OSITECH_ISR);
 	outb(0x03, ioaddr - 0x10 + OSITECH_ISR);
     }
+#ifdef DOES_NOT_WORK
+    if (lp->base != NULL) { /* Megahertz MFC's */
+	readb(lp->base+MEGAHERTZ_ISR);
+	readb(lp->base+MEGAHERTZ_ISR);
+    }
+#endif
 }
 
 /*====================================================================*/
@@ -1576,7 +1672,7 @@ static void smc_reset(struct device *dev)
 
 /*====================================================================*/
 
-int init_smc91c92_cs(void)
+int init_module(void)
 {
     servinfo_t serv;
 #ifdef PCMCIA_DEBUG
@@ -1593,7 +1689,6 @@ int init_smc91c92_cs(void)
     return 0;
 }
 
-#ifdef MODULE
 void cleanup_module(void)
 {
 #ifdef PCMCIA_DEBUG
@@ -1604,4 +1699,3 @@ void cleanup_module(void)
     while (dev_list != NULL)
 	smc91c92_detach(dev_list);
 }
-#endif /* MODULE */
