@@ -6,12 +6,8 @@
     
 ======================================================================*/
 
-#include "config.h"
-#include "k_compat.h"
-
-#ifdef MODULE
-#define init_tcic init_module
-#endif
+#include <pcmcia/config.h>
+#include <pcmcia/k_compat.h>
 
 #include <sys/types.h>
 #include <linux/fcntl.h>
@@ -29,16 +25,16 @@
 #include <linux/timer.h>
 #include <linux/ioport.h>
 
-#include "version.h"
-#include "ss.h"
+#include <pcmcia/version.h>
+#include <pcmcia/ss.h>
+#include <pcmcia/cs_types.h>
+#include <pcmcia/cs.h>
 #include "tcic.h"
-#include "cs_types.h"
-#include "cs.h"
 
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
 static const char *version =
-"tcic.c 1.75 1996/08/29 04:43:49 (David Hinds)";
+"tcic.c 1.78 1996/12/21 19:31:00 (David Hinds)";
 #endif
 
 /*====================================================================*/
@@ -416,7 +412,7 @@ int tcic_init(void)
     if (poll_interval == 0) {
 	if (cs_irq == 0) {
 	    for (i = 15; i > 0; i--)
-		if ((mask & (1 << i)) &&
+		if ((i != 12) && (mask & (1 << i)) &&
 		    (REQUEST_IRQ(i, tcic_interrupt, 0, "tcic") == 0))
 		    break;
 	    cs_irq = i;
@@ -958,7 +954,7 @@ static int tcic_service(u_int lsock, u_int cmd, void *arg)
 
 /*====================================================================*/
 
-int init_tcic(void)
+int init_module(void)
 {
     servinfo_t serv;
     CardServices(GetCardServicesInfo, &serv);
@@ -970,9 +966,7 @@ int init_tcic(void)
     return tcic_init();
 }
 
-#ifdef MODULE
 void cleanup_module(void)
 {
     tcic_finish();
 }
-#endif /* MODULE */
