@@ -13,14 +13,8 @@
 #undef XIRCNW_STATS
 #define PCMCIA_DEBUG 0
 
-#include "config.h"
-#include "k_compat.h"
-
-#ifdef MODULE
-#define init_xircnw init_module
-#include <linux/module.h>
-#include <linux/version.h>
-#endif
+#include <pcmcia/config.h>
+#include <pcmcia/k_compat.h>
 
 #include <assert.h>
 #include <linux/kernel.h>
@@ -44,12 +38,13 @@
 #include <linux/etherdevice.h>
 #include <linux/skbuff.h>
 
-#include "version.h"
-#include "cs_types.h"
-#include "cs.h"
-#include "cistpl.h"
-#include "cisreg.h"
-#include "ds.h"
+#include <pcmcia/version.h>
+#include <pcmcia/cs_types.h>
+#include <pcmcia/cs.h>
+#include <pcmcia/cistpl.h>
+#include <pcmcia/cisreg.h>
+#include <pcmcia/ds.h>
+#include <pcmcia/mem_op.h>
 
 #define XIRCOM_REGOFF         0x8000
 /* The Xircom IO registers, offsets to iobase */
@@ -257,13 +252,13 @@ static struct enet_statistics *xircnw_get_stats(struct device *dev);
 
 inline unsigned short get_uint16(u_char* staddr)
 {
-    return *((unsigned short*) staddr) & 0xffff; /* Return only 16 bits */
+    return readw(staddr); /* Return only 16 bits */
 }
 
 inline short get_int16(u_char* staddr)
 {
     assert(2 == sizeof(short));
-    return *((short*) staddr);
+    return readw(staddr);
 }
 
 /*====================================================================*/
@@ -594,23 +589,24 @@ static void xircnw_config(dev_link_t *link)
 
     /* Read the ethernet address and fill in the Netwave registers. */
     for (i = 0; i < 6; i++) 
-	dev->dev_addr[i] = ramBase[XIRCOM_EREG_PA + i];
+	dev->dev_addr[i] = readb(ramBase + XIRCOM_EREG_PA + i);
 
     if (xinw_debug) {
         printk(KERN_DEBUG "xircnw_config: %s irq %d, io %lx\n", 
 	       dev->name, dev->irq, dev->base_addr);
         printk(KERN_DEBUG "xircnw_config: memory base is %lx  id %c%c\n",
 	       (u_long) ramBase, 
-	       ramBase[XIRCOM_EREG_NI], ramBase[XIRCOM_EREG_NI+1]); 
+	       readb(ramBase+XIRCOM_EREG_NI),
+	       readb(ramBase+XIRCOM_EREG_NI+1));
     }
 
     printk(KERN_DEBUG "xircnw_config: phys addr: %02x:%02x:%02x:%02x:%02x:%02x\n", 
-	   ramBase[XIRCOM_EREG_PA+0], 
-	   ramBase[XIRCOM_EREG_PA+1], 
-	   ramBase[XIRCOM_EREG_PA+2], 
-	   ramBase[XIRCOM_EREG_PA+3], 
-	   ramBase[XIRCOM_EREG_PA+4], 
-	   ramBase[XIRCOM_EREG_PA+5]); 
+	   readb(ramBase+XIRCOM_EREG_PA+0),
+	   readb(ramBase+XIRCOM_EREG_PA+1),
+	   readb(ramBase+XIRCOM_EREG_PA+2),
+	   readb(ramBase+XIRCOM_EREG_PA+3),
+	   readb(ramBase+XIRCOM_EREG_PA+4),
+	   readb(ramBase+XIRCOM_EREG_PA+5));
 
     /* get revision words */
     printk(KERN_DEBUG "Xircom_reset: revision %04x %04x\n", 
@@ -748,7 +744,7 @@ static void xinw_doreset(unsigned long ioBase, u_char* ramBase)
 {
     /* Reset card */
     outb(0x80, ioBase + XIRCOM_REG_PMR);
-    ramBase[XIRCOM_EREG_ASCC] = 0x08;   /* Bit 3 is WOC */
+    writeb(0x08, ramBase + XIRCOM_EREG_ASCC); /* Bit 3 is WOC */
     outb(0x0, ioBase + XIRCOM_REG_PMR); /* release reset */
     wait_WOC(ioBase);
 }
@@ -769,37 +765,37 @@ static void xircnw_reset(struct device *dev)
     printk(KERN_DEBUG "xircnw_reset: Done with hardware reset\n");
    
     /* Write a NOP to check the card */
-    ramBase[XIRCOM_EREG_CB + 0] = XIRCOM_CMD_NOP;
-    ramBase[XIRCOM_EREG_CB + 1] = XIRCOM_CMD_EOC;
+    writeb(XIRCOM_CMD_NOP, ramBase + XIRCOM_EREG_CB + 0);
+    writeb(XIRCOM_CMD_EOC, ramBase + XIRCOM_EREG_CB + 1);
     wait_WOC(iobase);
 
     /* Set receive conf */
-    ramBase[XIRCOM_EREG_CB + 0] = XIRCOM_CMD_SRC;
-    ramBase[XIRCOM_EREG_CB + 1] = rxConfRxEna + rxConfBcast;
-    ramBase[XIRCOM_EREG_CB + 2] = XIRCOM_CMD_EOC;
+    writeb(XIRCOM_CMD_SRC, ramBase + XIRCOM_EREG_CB + 0);
+    writeb(rxConfRxEna + rxConfBcast, ramBase + XIRCOM_EREG_CB + 1);
+    writeb(XIRCOM_CMD_EOC, ramBase + XIRCOM_EREG_CB + 2);
     wait_WOC(iobase);
     
     /* Set transmit conf */
-    ramBase[XIRCOM_EREG_CB + 0] = XIRCOM_CMD_STC;
-    ramBase[XIRCOM_EREG_CB + 1] = txConfTxEna;
-    ramBase[XIRCOM_EREG_CB + 2] = XIRCOM_CMD_EOC;
+    writeb(XIRCOM_CMD_STC, ramBase + XIRCOM_EREG_CB + 0);
+    writeb(txConfTxEna, ramBase + XIRCOM_EREG_CB + 1);
+    writeb(XIRCOM_CMD_EOC, ramBase + XIRCOM_EREG_CB + 2);
     wait_WOC(iobase);
     
 
     /* Now set the MU Domain */
     printk(KERN_DEBUG "Setting domain to 0x%x%02x\n", (domain >> 8) & 0x01, domain & 0xff);
-    ramBase[XIRCOM_EREG_CB + 0] = XIRCOM_CMD_SMD;
-    ramBase[XIRCOM_EREG_CB + 1] = domain & 0xff; /* LSB */
-    ramBase[XIRCOM_EREG_CB + 2] = (domain >> 8) & 0x01; /* MSB */
-    ramBase[XIRCOM_EREG_CB + 3] = XIRCOM_CMD_EOC;
+    writeb(XIRCOM_CMD_SMD, ramBase + XIRCOM_EREG_CB + 0);
+    writeb(domain & 0xff, ramBase + XIRCOM_EREG_CB + 1);
+    writeb((domain>>8) & 0x01, ramBase + XIRCOM_EREG_CB + 2);
+    writeb(XIRCOM_CMD_EOC, ramBase + XIRCOM_EREG_CB + 3);
     wait_WOC(iobase);
 
     /* Set scramble key */
     printk(KERN_DEBUG "Setting scramble key to 0x%x\n", scramble_key);
-    ramBase[XIRCOM_EREG_CB + 0] = XIRCOM_CMD_SSK;
-    ramBase[XIRCOM_EREG_CB + 1] = scramble_key & 0xff; /* LSB */
-    ramBase[XIRCOM_EREG_CB + 2] = (scramble_key >> 8) & 0x01; /* MSB */
-    ramBase[XIRCOM_EREG_CB + 3] = XIRCOM_CMD_EOC;
+    writeb(XIRCOM_CMD_SSK, ramBase + XIRCOM_EREG_CB + 0);
+    writeb(scramble_key & 0xff, ramBase + XIRCOM_EREG_CB + 1);
+    writeb((scramble_key>>8) & 0x01, ramBase + XIRCOM_EREG_CB + 2);
+    writeb(XIRCOM_CMD_EOC, ramBase + XIRCOM_EREG_CB + 3);
     wait_WOC(iobase);
 
 
@@ -819,8 +815,8 @@ static void xircnw_reset(struct device *dev)
      */
 
     /* Enable Receiver */
-    ramBase[XIRCOM_EREG_CB + 0] = XIRCOM_CMD_ER;
-    ramBase[XIRCOM_EREG_CB + 1] = XIRCOM_CMD_EOC;
+    writeb(XIRCOM_CMD_ER, ramBase + XIRCOM_EREG_CB + 0);
+    writeb(XIRCOM_CMD_EOC, ramBase + XIRCOM_EREG_CB + 1);
     wait_WOC(iobase);
 
     /* Set the IENA bit in COR */
@@ -856,10 +852,10 @@ static int xircnw_hw_xmit(unsigned char* data, int len, struct device* dev)
 
     if (xinw_debug > 4) {
 	printk(KERN_DEBUG "Transmitting with SPCQ %x SPU %x LIF %x ISPLQ %x\n",
-	       ramBase[XIRCOM_EREG_SPCQ],
-	       ramBase[XIRCOM_EREG_SPU],
-	       ramBase[XIRCOM_EREG_LIF],
-	       ramBase[XIRCOM_EREG_ISPLQ]);
+	       readb(ramBase + XIRCOM_EREG_SPCQ),
+	       readb(ramBase + XIRCOM_EREG_SPU),
+	       readb(ramBase + XIRCOM_EREG_LIF),
+	       readb(ramBase + XIRCOM_EREG_ISPLQ));
     }
 
     wait_WOC(iobase);
@@ -876,14 +872,14 @@ static int xircnw_hw_xmit(unsigned char* data, int len, struct device* dev)
 	       TxFreeList, MaxData, DataOffset, nextBuff);
 
 
-    if (len <= MaxData) 
-	memcpy(ramBase + TxFreeList + DataOffset, data, len);
+    if (len <= MaxData)
+	copy_to_pc(ramBase + TxFreeList + DataOffset, data, len);
     else {
 	int tmplen = len;
 	int tmpcount = 0;
 	while (tmplen > 0) {
-	    memcpy(ramBase + curBuff + DataOffset, data + tmpcount, 
-		   (tmplen < MaxData) ? tmplen : MaxData);
+	    copy_to_pc(ramBase + curBuff + DataOffset, data + tmpcount, 
+		       (tmplen < MaxData) ? tmplen : MaxData);
 	    tmplen -= MaxData;
 	    tmpcount += MaxData;
 
@@ -895,10 +891,10 @@ static int xircnw_hw_xmit(unsigned char* data, int len, struct device* dev)
     
     /* Now issue transmit list */
     wait_WOC(iobase);
-    ramBase[XIRCOM_EREG_CB + 0] = XIRCOM_CMD_TL;
-    ramBase[XIRCOM_EREG_CB + 1] = len & 0xff;
-    ramBase[XIRCOM_EREG_CB + 2] = (len >> 8) & 0xff;
-    ramBase[XIRCOM_EREG_CB + 3] = XIRCOM_CMD_EOC;
+    writeb(XIRCOM_CMD_TL, ramBase + XIRCOM_EREG_CB + 0);
+    writeb(len & 0xff, ramBase + XIRCOM_EREG_CB + 1);
+    writeb((len>>8) & 0xff, ramBase + XIRCOM_EREG_CB + 2);
+    writeb(XIRCOM_CMD_EOC, ramBase + XIRCOM_EREG_CB + 3);
 
     return 0;
 }
@@ -1018,7 +1014,7 @@ static void xinw_interrupt IRQ(int irq, void* dev_id, struct pt_regs *regs)
 	    u_char rser;
 	   
 	    wait_WOC(iobase);
-	    rser = ramBase[XIRCOM_EREG_RSER];
+	    rser = readb(ramBase + XIRCOM_EREG_RSER);
 	
 	   
 	    if (rser & 0x04) {
@@ -1031,13 +1027,13 @@ static void xinw_interrupt IRQ(int irq, void* dev_id, struct pt_regs *regs)
 	    /* Clear the RxErr bit in RSER. RSER+4 is the
 	     * write part. Also clear the RxCRC (0x04) and 
 	     * RxBig (0x02) bits if present */
-	    ramBase[XIRCOM_EREG_RSER+4] = 0x40 | (rser & 0x06);
+	    writeb(0x40 | (rser & 0x06), ramBase + XIRCOM_EREG_RSER + 4);
 
 	    /* Write bit 6 high to ASCC to clear RxErr in ASR,
 	     * WOC must be set first! 
 	     */
 	    wait_WOC(iobase);
-	    ramBase[XIRCOM_EREG_ASCC] = 0x40; 
+	    writeb(0x40, ramBase + XIRCOM_EREG_ASCC);
 	    /* wait_WOC(iobase); */
 
 	    /* Remember to count up priv->stats on error packets */
@@ -1047,15 +1043,15 @@ static void xinw_interrupt IRQ(int irq, void* dev_id, struct pt_regs *regs)
 	if (status & 0x20) {
 	    int txStatus;
 	    wait_WOC(iobase);
-	    txStatus = ramBase[XIRCOM_EREG_TSER];
+	    txStatus = readb(ramBase + XIRCOM_EREG_TSER);
 	    if (xinw_debug > 4) 
 		printk(KERN_DEBUG "Transmit done. TSER = %x id %x\n", 
-		       txStatus, ramBase[XIRCOM_EREG_TSER+1]);
+		       txStatus, readb(ramBase + XIRCOM_EREG_TSER + 1));
 
 	    if (txStatus & 0x20) {
 		/* Transmitting was okay, clear bits */
 		wait_WOC(iobase);
-		ramBase[XIRCOM_EREG_TSER+4] = 0x2f;
+		writeb(0x2f, ramBase + XIRCOM_EREG_TSER + 4);
 	        ++priv->stats.tx_packets;
 	    }
 	    
@@ -1079,14 +1075,14 @@ static void xinw_interrupt IRQ(int irq, void* dev_id, struct pt_regs *regs)
 
 		/* Clear out TxGU, TxNOAP, TxErr and TxTrys */
 		wait_WOC(iobase);
-		ramBase[XIRCOM_EREG_TSER+4] = 0xdf & txStatus;
+		writeb(0xdf & txStatus, ramBase+XIRCOM_EREG_TSER+4);
 	        ++priv->stats.tx_errors;
 	    }
 	    /* wait_WOC(iobase); */
 	    
 	    if (xinw_debug > 4) 
 		printk(KERN_DEBUG "New status is TSER %x ASR %x\n",
-		       ramBase[XIRCOM_EREG_TSER],
+		       readb(ramBase + XIRCOM_EREG_TSER),
 		       inb(iobase + XIRCOM_REG_ASR));
 
 	    dev->tbusy = 0;
@@ -1118,17 +1114,17 @@ static struct enet_statistics *xinw_get_stats(struct device *dev)
 
     if (xinw_debug > 1) 
 	printk(KERN_DEBUG "xircom: SPCQ %x SPU %x LIF %x ISPLQ %x MHS %x rxtx %x %x tx %x %x %x %x\n", 
-	       priv->ramBase[XIRCOM_EREG_SPCQ],
-	       priv->ramBase[XIRCOM_EREG_SPU],
-	       priv->ramBase[XIRCOM_EREG_LIF],
-	       priv->ramBase[XIRCOM_EREG_ISPLQ],
-	       priv->ramBase[XIRCOM_EREG_MHS],
-	       priv->ramBase[XIRCOM_EREG_EC + 0xe],
-	       priv->ramBase[XIRCOM_EREG_EC + 0xf],
-	       priv->ramBase[XIRCOM_EREG_EC + 0x18],
-	       priv->ramBase[XIRCOM_EREG_EC + 0x19],
-	       priv->ramBase[XIRCOM_EREG_EC + 0x1a],
-	       priv->ramBase[XIRCOM_EREG_EC + 0x1b]
+	       readb(priv->ramBase + XIRCOM_EREG_SPCQ),
+	       readb(priv->ramBase + XIRCOM_EREG_SPU),
+	       readb(priv->ramBase + XIRCOM_EREG_LIF),
+	       readb(priv->ramBase + XIRCOM_EREG_ISPLQ),
+	       readb(priv->ramBase + XIRCOM_EREG_MHS),
+	       readb(priv->ramBase + XIRCOM_EREG_EC + 0xe),
+	       readb(priv->ramBase + XIRCOM_EREG_EC + 0xf),
+	       readb(priv->ramBase + XIRCOM_EREG_EC + 0x18),
+	       readb(priv->ramBase + XIRCOM_EREG_EC + 0x19),
+	       readb(priv->ramBase + XIRCOM_EREG_EC + 0x1a),
+	       readb(priv->ramBase + XIRCOM_EREG_EC + 0x1b)
 	    );
     return &priv->stats;
 }
@@ -1136,8 +1132,8 @@ static struct enet_statistics *xinw_get_stats(struct device *dev)
 static void update_stats(struct device *dev)
 {
 /*     xircom_private *priv = (xircom_private*) dev->priv;
-    priv->stats.rx_packets = priv->ramBase[0x18e]; 
-    priv->stats.tx_packets = priv->ramBase[0x18f]; */
+    priv->stats.rx_packets = readb(priv->ramBase + 0x18e); 
+    priv->stats.tx_packets = readb(priv->ramBase + 0x18f); */
 }
 
 static int xinw_rx(struct device *dev)
@@ -1168,7 +1164,7 @@ static int xinw_rx(struct device *dev)
     /* Receive max 10 packets for now. */
     for (i = 0; i < 10; i++) {
 	wait_WOC(iobase);
-	rxStatus = ramBase[XIRCOM_EREG_RSER];
+	rxStatus = readb(ramBase + XIRCOM_EREG_RSER);
       
 	if (!(rxStatus & 0x80)) /* No more packets */
 	    break;
@@ -1216,14 +1212,14 @@ static int xinw_rx(struct device *dev)
 	ptr = (u_char*) skb->data;
       
 	if (rcvLen <= maxData) 
-	    memcpy(ptr, ramBase + rcvList + dataOffset + extraOffset, rcvLen);
+	    copy_from_pc(ptr, ramBase+rcvList+dataOffset+extraOffset, rcvLen);
 	else {
 	    int tmplen = rcvLen;
 	    int tmpcount = 0;
 	    while (tmplen > 0) {
-		memcpy(ptr + tmpcount ,
-		       ramBase + curBuffer + dataOffset + extraOffset,
-		       (tmplen < maxData) ? tmplen : maxData);
+		copy_from_pc(ptr + tmpcount ,
+			     ramBase + curBuffer + dataOffset + extraOffset,
+			     (tmplen < maxData) ? tmplen : maxData);
 		tmplen -= maxData;
 		tmpcount += maxData;
 	       
@@ -1249,8 +1245,8 @@ static int xinw_rx(struct device *dev)
 
 	wait_WOC(iobase);
 	/* Got the packet, tell the adapter to skip it */
-	ramBase[XIRCOM_EREG_CB + 0] = XIRCOM_CMD_SRP;
-	ramBase[XIRCOM_EREG_CB + 1] = XIRCOM_CMD_EOC;
+	writeb(XIRCOM_CMD_SRP, ramBase + XIRCOM_EREG_CB + 0);
+	writeb(XIRCOM_CMD_EOC, ramBase + XIRCOM_EREG_CB + 1);
       
 	if (xinw_debug > 2) 
 	    printk(KERN_DEBUG "Packet reception ok\n");
@@ -1305,7 +1301,7 @@ static int xinw_close(struct device *dev)
     return 0;
 }
 
-int init_xircnw(void)
+int init_module(void)
 {
     servinfo_t serv;
 	
@@ -1321,18 +1317,8 @@ int init_xircnw(void)
     return 0;
 }
 
-#ifdef MODULE
-/* char kernel_version[] = UTS_RELEASE; */
 void cleanup_module(void)
 {
-/* 
-    if (MOD_IN_USE)
-        printk("xircnw: device busy, remove delayed\n");
-    else {
-        unregister_netdev(&dev_xircnw);
-    }
-
-    */
     printk("xircnw_cs: unloading\n");
     unregister_pcmcia_driver(&dev_info);
     while (dev_list != NULL) {
@@ -1341,16 +1327,6 @@ void cleanup_module(void)
 	xircnw_detach(dev_list);
     }
 }
-#endif /* MODULE */
-
-/*
-static int xinw_probe(struct device *dev)
-{
-    xircnw_init(dev);
-    return 0;
-}
-*/
-
 
 
 /* Set or clear the multicast filter for this adaptor.
@@ -1388,9 +1364,9 @@ static void set_multicast_list(struct device *dev)
    
     printk("xircom set_multicast_list: rcvMode to %x\n", rcvMode);
     /* Now set receive mode */
-    ramBase[XIRCOM_EREG_CB + 0] = XIRCOM_CMD_SRC;
-    ramBase[XIRCOM_EREG_CB + 1] = rcvMode;
-    ramBase[XIRCOM_EREG_CB + 2] = XIRCOM_CMD_EOC;
+    writeb(XIRCOM_CMD_SRC, ramBase + XIRCOM_EREG_CB + 0);
+    writeb(rcvMode, ramBase + XIRCOM_EREG_CB + 1);
+    writeb(XIRCOM_CMD_EOC, ramBase + XIRCOM_EREG_CB + 2);
     wait_WOC(iobase);
 }
 #else
@@ -1423,9 +1399,9 @@ set_multicast_list(struct device *dev, int num_addrs, void *addrs)
 
     printk("xircom set_multicast_list: rcvMode to %x\n", rcvMode);
     /* Now set receive mode */
-    ramBase[XIRCOM_EREG_CB + 0] = XIRCOM_CMD_SRC;
-    ramBase[XIRCOM_EREG_CB + 1] = rcvMode;
-    ramBase[XIRCOM_EREG_CB + 2] = XIRCOM_CMD_EOC;
+    writeb(XIRCOM_CMD_SRC, ramBase + XIRCOM_EREG_CB + 0);
+    writeb(rcvMode, ramBase + XIRCOM_EREG_CB + 1);
+    writeb(XIRCOM_CMD_EOC, ramBase + XIRCOM_EREG_CB + 2);
     wait_WOC(iobase);
 }
 #endif
